# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'h5viewerUI.ui'
##
## Created by: Qt User Interface Compiler version 6.10.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QFrame, QGridLayout,
    QLabel, QListView, QListWidget, QListWidgetItem,
    QPushButton, QSizePolicy, QSplitter, QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(1440, 842)
        Form.setMinimumSize(QSize(0, 0))
        self.gridLayout = QGridLayout(Form)
#ifndef Q_OS_MAC
        self.gridLayout.setSpacing(-1)
#endif
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(8, 8, 8, 12)
        self.label_3 = QLabel(Form)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setMaximumSize(QSize(16777215, 20))

        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)

        self.label_4 = QLabel(Form)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setMaximumSize(QSize(70, 16777215))
        self.label_4.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label_4, 1, 1, 1, 1)

        self.show_all = QPushButton(Form)
        self.show_all.setObjectName(u"show_all")
        self.show_all.setMaximumSize(QSize(16777215, 25))

        self.gridLayout.addWidget(self.show_all, 3, 0, 1, 1)

        self.auto_last = QPushButton(Form)
        self.auto_last.setObjectName(u"auto_last")
        self.auto_last.setMaximumSize(QSize(16777215, 25))

        self.gridLayout.addWidget(self.auto_last, 3, 1, 1, 1)

        self.splitter = QSplitter(Form)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setMinimumSize(QSize(0, 0))
        self.splitter.setOrientation(Qt.Horizontal)
        self.listScans = QListWidget(self.splitter)
        self.listScans.setObjectName(u"listScans")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listScans.sizePolicy().hasHeightForWidth())
        self.listScans.setSizePolicy(sizePolicy)
        self.listScans.setMinimumSize(QSize(30, 0))
        self.listScans.setResizeMode(QListView.Adjust)
        self.splitter.addWidget(self.listScans)
        self.listData = QListWidget(self.splitter)
        self.listData.setObjectName(u"listData")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.listData.sizePolicy().hasHeightForWidth())
        self.listData.setSizePolicy(sizePolicy1)
        self.listData.setMinimumSize(QSize(45, 0))
        self.listData.setMaximumSize(QSize(60, 16777215))
        self.listData.setFocusPolicy(Qt.StrongFocus)
        self.listData.setTabKeyNavigation(False)
        self.listData.setAlternatingRowColors(True)
        self.listData.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.splitter.addWidget(self.listData)

        self.gridLayout.addWidget(self.splitter, 2, 0, 1, 2)

        self.frame = QFrame(Form)
        self.frame.setObjectName(u"frame")
        self.frame.setMaximumSize(QSize(16777215, 20))
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)

        self.gridLayout.addWidget(self.frame, 0, 0, 1, 2)


        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Form", None))
        self.label_3.setText(QCoreApplication.translate("Form", u"  Scans", None))
        self.label_4.setText(QCoreApplication.translate("Form", u"Data", None))
        self.show_all.setText(QCoreApplication.translate("Form", u"Show All", None))
        self.auto_last.setText(QCoreApplication.translate("Form", u"Auto Last", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'displayFrameUI.ui'
##
## Created by: Qt User Interface Compiler version 6.10.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QDoubleSpinBox,
    QFrame, QHBoxLayout, QLabel, QPushButton,
    QSizePolicy, QSpacerItem, QSplitter, QVBoxLayout,
    QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(1440, 814)
        self.layout = QHBoxLayout(Form)
        self.layout.setSpacing(0)
        self.layout.setObjectName(u"layout")
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.splitter = QSplitter(Form)
        self.splitter.setObjectName(u"splitter")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setOrientation(Qt.Vertical)
        self.splitter.setHandleWidth(2)
        self.imageWindow = QFrame(self.splitter)
        self.imageWindow.setObjectName(u"imageWindow")
        sizePolicy.setHeightForWidth(self.imageWindow.sizePolicy().hasHeightForWidth())
        self.imageWindow.setSizePolicy(sizePolicy)
        self.imageWindow.setMinimumSize(QSize(0, 400))
        self.imageWindow.setFrameShape(QFrame.StyledPanel)
        self.imageWindow.setFrameShadow(QFrame.Raised)
        self.imageWindow.setLineWidth(3)
        self.verticalLayout_3 = QVBoxLayout(self.imageWindow)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.frame_top = QFrame(self.imageWindow)
        self.frame_top.setObjectName(u"frame_top")
        self.frame_top.setMinimumSize(QSize(0, 35))
        self.frame_top.setMaximumSize(QSize(16777215, 35))
        self.frame_top.setFrameShape(QFrame.StyledPanel)
        self.frame_top.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_5 = QHBoxLayout(self.frame_top)
        self.horizontalLayout_5.setSpacing(0)
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.frame_4 = QFrame(self.frame_top)
        self.frame_4.setObjectName(u"frame_4")
        self.frame_4.setMinimumSize(QSize(260, 0))
        self.frame_4.setMaximumSize(QSize(260, 16777215))
        self.frame_4.setFrameShape(QFrame.NoFrame)
        self.frame_4.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_8 = QHBoxLayout(self.frame_4)
        self.horizontalLayout_8.setSpacing(0)
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.horizontalLayout_8.setContentsMargins(10, 0, 0, 0)
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.normChannel = QComboBox(self.frame_4)
        self.normChannel.addItem("")
        self.normChannel.addItem("")
        self.normChannel.addItem("")
        self.normChannel.addItem("")
        self.normChannel.addItem("")
        self.normChannel.addItem("")
        self.normChannel.setObjectName(u"normChannel")
        self.normChannel.setMinimumSize(QSize(135, 0))
        self.normChannel.setMaximumSize(QSize(140, 16777215))
        self.normChannel.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout_7.addWidget(self.normChannel)

        self.horizontalSpacer_3 = QSpacerItem(15, 20, QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_7.addItem(self.horizontalSpacer_3)

        self.setBkg = QPushButton(self.frame_4)
        self.setBkg.setObjectName(u"setBkg")
        self.setBkg.setMinimumSize(QSize(90, 0))
        self.setBkg.setMaximumSize(QSize(100, 16777215))
        self.setBkg.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout_7.addWidget(self.setBkg)

        self.horizontalSpacer_11 = QSpacerItem(10, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_7.addItem(self.horizontalSpacer_11)


        self.horizontalLayout_8.addLayout(self.horizontalLayout_7)


        self.horizontalLayout_5.addWidget(self.frame_4)

        self.frame_5 = QFrame(self.frame_top)
        self.frame_5.setObjectName(u"frame_5")
        self.frame_5.setFrameShape(QFrame.NoFrame)
        self.frame_5.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_6 = QHBoxLayout(self.frame_5)
        self.horizontalLayout_6.setSpacing(0)
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.labelCurrent = QLabel(self.frame_5)
        self.labelCurrent.setObjectName(u"labelCurrent")
        self.labelCurrent.setMaximumSize(QSize(600, 16777215))
        font = QFont()
        font.setPointSize(15)
        self.labelCurrent.setFont(font)
        self.labelCurrent.setAlignment(Qt.AlignCenter)

        self.horizontalLayout_6.addWidget(self.labelCurrent)


        self.horizontalLayout_5.addWidget(self.frame_5)

        self.frame_6 = QFrame(self.frame_top)
        self.frame_6.setObjectName(u"frame_6")
        self.frame_6.setMinimumSize(QSize(260, 0))
        self.frame_6.setMaximumSize(QSize(260, 16777215))
        self.frame_6.setFrameShape(QFrame.NoFrame)
        self.frame_6.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_10 = QHBoxLayout(self.frame_6)
        self.horizontalLayout_10.setSpacing(0)
        self.horizontalLayout_10.setObjectName(u"horizontalLayout_10")
        self.horizontalLayout_10.setContentsMargins(0, 0, 10, 0)
        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setSpacing(0)
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.horizontalSpacer_12 = QSpacerItem(20, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_9.addItem(self.horizontalSpacer_12)

        self.scale = QComboBox(self.frame_6)
        self.scale.addItem("")
        self.scale.addItem("")
        self.scale.addItem("")
        self.scale.addItem("")
        self.scale.setObjectName(u"scale")
        self.scale.setMinimumSize(QSize(80, 0))
        self.scale.setMaximumSize(QSize(80, 16777215))
        self.scale.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout_9.addWidget(self.scale)

        self.horizontalSpacer_10 = QSpacerItem(15, 20, QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_9.addItem(self.horizontalSpacer_10)

        self.cmap = QComboBox(self.frame_6)
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.addItem("")
        self.cmap.setObjectName(u"cmap")
        self.cmap.setMinimumSize(QSize(80, 0))
        self.cmap.setMaximumSize(QSize(80, 16777215))
        self.cmap.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout_9.addWidget(self.cmap)


        self.horizontalLayout_10.addLayout(self.horizontalLayout_9)


        self.horizontalLayout_5.addWidget(self.frame_6)


        self.verticalLayout_3.addWidget(self.frame_top)

        self.twoDWindow = QFrame(self.imageWindow)
        self.twoDWindow.setObjectName(u"twoDWindow")
        self.twoDWindow.setFrameShape(QFrame.StyledPanel)
        self.twoDWindow.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_3 = QHBoxLayout(self.twoDWindow)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.splitter_2 = QSplitter(self.twoDWindow)
        self.splitter_2.setObjectName(u"splitter_2")
        self.splitter_2.setOrientation(Qt.Horizontal)
        self.splitter_2.setHandleWidth(2)
        self.imageFrame = QFrame(self.splitter_2)
        self.imageFrame.setObjectName(u"imageFrame")
        self.imageFrame.setFrameShape(QFrame.StyledPanel)
        self.imageFrame.setFrameShadow(QFrame.Raised)
        self.splitter_2.addWidget(self.imageFrame)
        self.binnedFrame = QFrame(self.splitter_2)
        self.binnedFrame.setObjectName(u"binnedFrame")
        self.binnedFrame.setFrameShape(QFrame.StyledPanel)
        self.binnedFrame.setFrameShadow(QFrame.Raised)
        self.splitter_2.addWidget(self.binnedFrame)

        self.horizontalLayout_3.addWidget(self.splitter_2)


        self.verticalLayout_3.addWidget(self.twoDWindow)

        self.imageToolbar = QFrame(self.imageWindow)
        self.imageToolbar.setObjectName(u"imageToolbar")
        self.imageToolbar.setMinimumSize(QSize(0, 40))
        self.imageToolbar.setMaximumSize(QSize(16777215, 40))
        self.imageToolbar.setFrameShape(QFrame.NoFrame)
        self.imageToolbar.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.imageToolbar)
        self.horizontalLayout_2.setSpacing(6)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(12, 0, -1, 0)
        self.imageUnit = QComboBox(self.imageToolbar)
        self.imageUnit.addItem("")
        self.imageUnit.addItem("")
        self.imageUnit.addItem("")
        self.imageUnit.setObjectName(u"imageUnit")
        self.imageUnit.setMinimumSize(QSize(90, 0))
        self.imageUnit.setMaximumSize(QSize(130, 16777215))
        self.imageUnit.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout_2.addWidget(self.imageUnit)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_2)

        self.update2D = QCheckBox(self.imageToolbar)
        self.update2D.setObjectName(u"update2D")
        self.update2D.setFocusPolicy(Qt.StrongFocus)
        self.update2D.setChecked(True)

        self.horizontalLayout_2.addWidget(self.update2D)

        self.horizontalSpacer_9 = QSpacerItem(40, 20, QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_9)

        self.shareAxis = QCheckBox(self.imageToolbar)
        self.shareAxis.setObjectName(u"shareAxis")
        self.shareAxis.setMaximumSize(QSize(90, 16777215))
        self.shareAxis.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout_2.addWidget(self.shareAxis)

        self.horizontalSpacer_14 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_14)

        self.save_2D = QPushButton(self.imageToolbar)
        self.save_2D.setObjectName(u"save_2D")

        self.horizontalLayout_2.addWidget(self.save_2D)


        self.verticalLayout_3.addWidget(self.imageToolbar)

        self.splitter.addWidget(self.imageWindow)
        self.plotWindow = QFrame(self.splitter)
        self.plotWindow.setObjectName(u"plotWindow")
        sizePolicy.setHeightForWidth(self.plotWindow.sizePolicy().hasHeightForWidth())
        self.plotWindow.setSizePolicy(sizePolicy)
        self.plotWindow.setMinimumSize(QSize(0, 400))
        self.plotWindow.setFrameShape(QFrame.StyledPanel)
        self.plotWindow.setFrameShadow(QFrame.Raised)
        self.plotWindow.setLineWidth(3)
        self.verticalLayout_4 = QVBoxLayout(self.plotWindow)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.oneDWindow = QFrame(self.plotWindow)
        self.oneDWindow.setObjectName(u"oneDWindow")
        self.oneDWindow.setFrameShape(QFrame.StyledPanel)
        self.oneDWindow.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_4 = QHBoxLayout(self.oneDWindow)
        self.horizontalLayout_4.setSpacing(0)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.splitter_3 = QSplitter(self.oneDWindow)
        self.splitter_3.setObjectName(u"splitter_3")
        self.splitter_3.setOrientation(Qt.Horizontal)
        self.splitter_3.setHandleWidth(2)
        self.plotFrame = QFrame(self.splitter_3)
        self.plotFrame.setObjectName(u"plotFrame")
        self.plotFrame.setFrameShape(QFrame.StyledPanel)
        self.plotFrame.setFrameShadow(QFrame.Raised)
        self.splitter_3.addWidget(self.plotFrame)

        self.horizontalLayout_4.addWidget(self.splitter_3)


        self.verticalLayout_4.addWidget(self.oneDWindow)

        self.plotToolBar = QFrame(self.plotWindow)
        self.plotToolBar.setObjectName(u"plotToolBar")
        self.plotToolBar.setMinimumSize(QSize(0, 40))
        self.plotToolBar.setMaximumSize(QSize(16777215, 40))
        self.plotToolBar.setFrameShape(QFrame.StyledPanel)
        self.plotToolBar.setFrameShadow(QFrame.Raised)
        self.horizontalLayout = QHBoxLayout(self.plotToolBar)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(12, 0, -1, 0)
        self.plotUnit = QComboBox(self.plotToolBar)
        self.plotUnit.addItem("")
        self.plotUnit.addItem("")
        self.plotUnit.addItem("")
        self.plotUnit.setObjectName(u"plotUnit")
        self.plotUnit.setMinimumSize(QSize(70, 0))
        self.plotUnit.setMaximumSize(QSize(100, 16777215))
        self.plotUnit.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout.addWidget(self.plotUnit)

        self.slice = QCheckBox(self.plotToolBar)
        self.slice.setObjectName(u"slice")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.slice.sizePolicy().hasHeightForWidth())
        self.slice.setSizePolicy(sizePolicy1)
        self.slice.setMinimumSize(QSize(80, 0))
        self.slice.setMaximumSize(QSize(80, 16777215))

        self.horizontalLayout.addWidget(self.slice)

        self.slice_center = QDoubleSpinBox(self.plotToolBar)
        self.slice_center.setObjectName(u"slice_center")
        sizePolicy1.setHeightForWidth(self.slice_center.sizePolicy().hasHeightForWidth())
        self.slice_center.setSizePolicy(sizePolicy1)
        self.slice_center.setMinimumSize(QSize(70, 0))
        self.slice_center.setMaximumSize(QSize(70, 16777215))
        self.slice_center.setToolTipDuration(2)
        self.slice_center.setMinimum(-180.000000000000000)
        self.slice_center.setMaximum(180.000000000000000)
        self.slice_center.setSingleStep(0.500000000000000)
        self.slice_center.setValue(0.000000000000000)

        self.horizontalLayout.addWidget(self.slice_center)

        self.slice_width = QDoubleSpinBox(self.plotToolBar)
        self.slice_width.setObjectName(u"slice_width")
        sizePolicy1.setHeightForWidth(self.slice_width.sizePolicy().hasHeightForWidth())
        self.slice_width.setSizePolicy(sizePolicy1)
        self.slice_width.setMinimumSize(QSize(70, 0))
        self.slice_width.setMaximumSize(QSize(70, 16777215))
        self.slice_width.setMinimum(0.000000000000000)
        self.slice_width.setMaximum(270.000000000000000)
        self.slice_width.setSingleStep(0.500000000000000)
        self.slice_width.setValue(5.000000000000000)

        self.horizontalLayout.addWidget(self.slice_width)

        self.horizontalSpacer_4 = QSpacerItem(80, 20, QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_4)

        self.plotMethod = QComboBox(self.plotToolBar)
        self.plotMethod.addItem("")
        self.plotMethod.addItem("")
        self.plotMethod.addItem("")
        self.plotMethod.addItem("")
        self.plotMethod.addItem("")
        self.plotMethod.setObjectName(u"plotMethod")
        self.plotMethod.setFocusPolicy(Qt.StrongFocus)

        self.horizontalLayout.addWidget(self.plotMethod)

        self.yOffsetLabel = QLabel(self.plotToolBar)
        self.yOffsetLabel.setObjectName(u"yOffsetLabel")
        self.yOffsetLabel.setMaximumSize(QSize(140, 16777215))

        self.horizontalLayout.addWidget(self.yOffsetLabel)

        self.yOffset = QDoubleSpinBox(self.plotToolBar)
        self.yOffset.setObjectName(u"yOffset")
        self.yOffset.setEnabled(False)
        self.yOffset.setFocusPolicy(Qt.StrongFocus)
        self.yOffset.setDecimals(1)
        self.yOffset.setMinimum(-100.000000000000000)
        self.yOffset.setSingleStep(5.000000000000000)
        self.yOffset.setValue(5.000000000000000)

        self.horizontalLayout.addWidget(self.yOffset)

        self.wf_options = QPushButton(self.plotToolBar)
        self.wf_options.setObjectName(u"wf_options")
        self.wf_options.setEnabled(False)

        self.horizontalLayout.addWidget(self.wf_options)

        self.horizontalSpacer_6 = QSpacerItem(30, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_6)

        self.showLegend = QCheckBox(self.plotToolBar)
        self.showLegend.setObjectName(u"showLegend")
        self.showLegend.setFocusPolicy(Qt.StrongFocus)
        self.showLegend.setChecked(True)

        self.horizontalLayout.addWidget(self.showLegend)

        self.clear_1D = QPushButton(self.plotToolBar)
        self.clear_1D.setObjectName(u"clear_1D")

        self.horizontalLayout.addWidget(self.clear_1D)

        self.save_1D = QPushButton(self.plotToolBar)
        self.save_1D.setObjectName(u"save_1D")

        self.horizontalLayout.addWidget(self.save_1D)


        self.verticalLayout_4.addWidget(self.plotToolBar)

        self.splitter.addWidget(self.plotWindow)

        self.layout.addWidget(self.splitter)

        QWidget.setTabOrder(self.normChannel, self.setBkg)
        QWidget.setTabOrder(self.setBkg, self.scale)
        QWidget.setTabOrder(self.scale, self.cmap)
        QWidget.setTabOrder(self.cmap, self.imageUnit)
        QWidget.setTabOrder(self.imageUnit, self.save_2D)
        QWidget.setTabOrder(self.save_2D, self.plotUnit)
        QWidget.setTabOrder(self.plotUnit, self.slice_center)
        QWidget.setTabOrder(self.slice_center, self.slice_width)
        QWidget.setTabOrder(self.slice_width, self.plotMethod)
        QWidget.setTabOrder(self.plotMethod, self.yOffset)
        QWidget.setTabOrder(self.yOffset, self.save_1D)

        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Form", None))
        self.normChannel.setItemText(0, QCoreApplication.translate("Form", u"Norm Channel", None))
        self.normChannel.setItemText(1, QCoreApplication.translate("Form", u"Monitor", None))
        self.normChannel.setItemText(2, QCoreApplication.translate("Form", u"sec", None))
        self.normChannel.setItemText(3, QCoreApplication.translate("Form", u"bstop", None))
        self.normChannel.setItemText(4, QCoreApplication.translate("Form", u"I0", None))
        self.normChannel.setItemText(5, QCoreApplication.translate("Form", u"I1", None))

        self.setBkg.setText(QCoreApplication.translate("Form", u"Set Bkg", None))
        self.labelCurrent.setText(QCoreApplication.translate("Form", u"Current", None))
        self.scale.setItemText(0, QCoreApplication.translate("Form", u"Linear", None))
        self.scale.setItemText(1, QCoreApplication.translate("Form", u"Log", None))
        self.scale.setItemText(2, QCoreApplication.translate("Form", u"Log-Log", None))
        self.scale.setItemText(3, QCoreApplication.translate("Form", u"Sqrt", None))

        self.cmap.setItemText(0, QCoreApplication.translate("Form", u"Default", None))
        self.cmap.setItemText(1, QCoreApplication.translate("Form", u"viridis", None))
        self.cmap.setItemText(2, QCoreApplication.translate("Form", u"grey", None))
        self.cmap.setItemText(3, QCoreApplication.translate("Form", u"plasma", None))
        self.cmap.setItemText(4, QCoreApplication.translate("Form", u"inferno", None))
        self.cmap.setItemText(5, QCoreApplication.translate("Form", u"magma", None))
        self.cmap.setItemText(6, QCoreApplication.translate("Form", u"thermal", None))
        self.cmap.setItemText(7, QCoreApplication.translate("Form", u"flame", None))
        self.cmap.setItemText(8, QCoreApplication.translate("Form", u"yellowy", None))
        self.cmap.setItemText(9, QCoreApplication.translate("Form", u"bipolar", None))
        self.cmap.setItemText(10, QCoreApplication.translate("Form", u"greyclip", None))

        self.imageUnit.setItemText(0, QCoreApplication.translate("Form", u"Q-Chi", None))
        self.imageUnit.setItemText(1, QCoreApplication.translate("Form", u"2Th-Chi", None))
        self.imageUnit.setItemText(2, QCoreApplication.translate("Form", u"Qz-Qxy", None))

        self.update2D.setText(QCoreApplication.translate("Form", u"Update 2D", None))
        self.shareAxis.setText(QCoreApplication.translate("Form", u"Share Axis", None))
        self.save_2D.setText(QCoreApplication.translate("Form", u"Save", None))
        self.plotUnit.setItemText(0, QCoreApplication.translate("Form", u"Q (A-1)", None))
        self.plotUnit.setItemText(1, QCoreApplication.translate("Form", u"2 u\\u03B8", None))
        self.plotUnit.setItemText(2, QCoreApplication.translate("Form", u"Chi", None))

        self.slice.setText(QCoreApplication.translate("Form", u"X Range", None))
#if QT_CONFIG(tooltip)
        self.slice_center.setToolTip("")
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        self.slice_center.setWhatsThis("")
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(accessibility)
        self.slice_center.setAccessibleDescription("")
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(tooltip)
        self.slice_width.setToolTip("")
#endif // QT_CONFIG(tooltip)
        self.plotMethod.setItemText(0, QCoreApplication.translate("Form", u"Single", None))
        self.plotMethod.setItemText(1, QCoreApplication.translate("Form", u"Overlay", None))
        self.plotMethod.setItemText(2, QCoreApplication.translate("Form", u"Average", None))
        self.plotMethod.setItemText(3, QCoreApplication.translate("Form", u"Sum", None))
        self.plotMethod.setItemText(4, QCoreApplication.translate("Form", u"Waterfall", None))

#if QT_CONFIG(tooltip)
        self.yOffsetLabel.setToolTip(QCoreApplication.translate("Form", u"y Offset for Overlay Mode", None))
#endif // QT_CONFIG(tooltip)
        self.yOffsetLabel.setText(QCoreApplication.translate("Form", u"Offset", None))
#if QT_CONFIG(tooltip)
        self.wf_options.setToolTip("")
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        self.wf_options.setWhatsThis("")
#endif // QT_CONFIG(whatsthis)
        self.wf_options.setText(QCoreApplication.translate("Form", u"Options", None))
        self.showLegend.setText(QCoreApplication.translate("Form", u"Legend", None))
        self.clear_1D.setText(QCoreApplication.translate("Form", u"Clear", None))
        self.save_1D.setText(QCoreApplication.translate("Form", u"Save", None))
    # retranslateUi




# -*- coding: utf-8 -*-
"""
nexusWrangler — GUI widget for processing NeXus/HDF5 image stacks.

Provides a simple interface for selecting a NeXus file containing
image frames (e.g., from Bluesky suitcase-nexus exports or Eiger
master files) and a PONI calibration file.

@author: thampy
"""

# Standard library imports
import os
from pathlib import Path

# Qt imports
from pyqtgraph.Qt import QtWidgets, QtCore
from pyqtgraph.parametertree import ParameterTree, Parameter

# Project imports
from ssrl_xrd_tools.core.containers import PONI
from .wrangler_widget import wranglerWidget
from .nexus_wrangler_thread import nexusThread
from ....gui_utils import NamedActionParameter
from xdart.utils import get_fname_dir
from xdart.utils.session import load_session, save_session

QFileDialog = QtWidgets.QFileDialog

params = [
    {'name': 'Calibration', 'type': 'group', 'children': [
        {'name': 'poni_file', 'title': 'PONI File    ', 'type': 'str', 'value': ''},
        NamedActionParameter(name='poni_file_browse', title='Browse...'),
    ], 'expanded': True},
    {'name': 'NeXus File', 'type': 'group', 'children': [
        {'name': 'nexus_file', 'title': 'File         ', 'type': 'str', 'value': ''},
        NamedActionParameter(name='nexus_file_browse', title='Browse...'),
        {'name': 'entry', 'title': 'Entry', 'type': 'str', 'value': 'entry'},
    ], 'expanded': True},
    {'name': 'Signal', 'type': 'group', 'children': [
        {'name': 'mask_file', 'title': 'Mask File    ', 'type': 'str', 'value': ''},
        NamedActionParameter(name='mask_file_browse', title='Browse...'),
    ], 'expanded': False},
    {'name': 'GI', 'type': 'group', 'children': [
        {'name': 'Grazing', 'title': 'Grazing Incidence', 'type': 'bool', 'value': False},
        {'name': 'th_motor', 'title': 'Theta Motor', 'type': 'str', 'value': 'th'},
        {'name': 'th_val', 'title': 'Theta Value', 'type': 'float', 'value': 0.0},
        {'name': 'sample_orientation', 'title': 'Sample Orientation', 'type': 'int',
         'value': 4, 'limits': (1, 4)},
        {'name': 'tilt_angle', 'title': 'Tilt Angle', 'type': 'float', 'value': 0.0},
        # gi_mode_1d / gi_mode_2d are controlled by the integrator panel;
        # kept here for session persistence only.
        {'name': 'gi_mode_1d', 'title': '1D Mode', 'type': 'list',
         'values': ['q_total', 'q_ip', 'q_oop', 'exit_angle'],
         'value': 'q_total', 'visible': False},
        {'name': 'gi_mode_2d', 'title': '2D Mode', 'type': 'list',
         'values': ['qip_qoop', 'q_chi', 'exit_angles'],
         'value': 'qip_qoop', 'visible': False},
    ], 'expanded': False},
    {'name': 'Output', 'type': 'group', 'children': [
        {'name': 'h5_dir', 'title': 'Output Dir   ', 'type': 'str', 'value': ''},
        NamedActionParameter(name='h5_dir_browse', title='Browse...'),
    ], 'expanded': False},
]


class nexusWrangler(wranglerWidget):
    """Widget for processing NeXus/HDF5 image stacks.

    A simpler alternative to specWrangler for data already stored
    in NeXus format (e.g. from Bluesky).

    signals:
        showLabel: str, status text
    """
    showLabel = QtCore.Signal(str)

    def __init__(self, fname, file_lock, sphere, data_1d, data_2d, parent=None):
        super().__init__(fname, file_lock, parent)

        self.poni = None
        self.command = None
        self.sphere = sphere
        self.data_1d = data_1d
        self.data_2d = data_2d

        # Attributes
        self.nexus_file = ''
        self.entry = 'entry'
        self.poni_file = ''
        self.mask_file = ''
        self.h5_dir = get_fname_dir()
        self.gi = False
        self.th_mtr = 'th'
        self.sample_orientation = 4
        self.tilt_angle = 0.0
        self.gi_mode_1d = 'q_total'
        self.gi_mode_2d = 'qip_qoop'

        # ── Build UI programmatically ────────────────────────────────
        layout = QtWidgets.QVBoxLayout(self)
        layout.setContentsMargins(2, 2, 2, 2)

        # Status label
        self.statusLabel = QtWidgets.QLabel('Ready')
        layout.addWidget(self.statusLabel)
        self.showLabel.connect(self.statusLabel.setText)

        # Buttons
        btn_layout = QtWidgets.QHBoxLayout()
        self.startButton = QtWidgets.QPushButton('Start')
        self.pauseButton = QtWidgets.QPushButton('Pause')
        self.stopButton = QtWidgets.QPushButton('Stop')
        self.continueButton = QtWidgets.QPushButton('Continue')
        self.skip2dCheckBox = QtWidgets.QCheckBox('Skip 2D')

        btn_layout.addWidget(self.startButton)
        btn_layout.addWidget(self.pauseButton)
        btn_layout.addWidget(self.stopButton)
        btn_layout.addWidget(self.continueButton)
        btn_layout.addWidget(self.skip2dCheckBox)
        layout.addLayout(btn_layout)

        self.startButton.clicked.connect(self.start)
        self.pauseButton.clicked.connect(self.pause)
        self.stopButton.clicked.connect(self.stop)
        self.continueButton.clicked.connect(self.cont)
        self.skip2dCheckBox.stateChanged.connect(
            lambda _: setattr(self.sphere, 'skip_2d', self.skip2dCheckBox.isChecked())
        )

        self.pauseButton.setEnabled(False)
        self.continueButton.setEnabled(False)
        self.stopButton.setEnabled(False)

        # Parameter tree
        self.tree = ParameterTree()
        self.parameters = Parameter.create(
            name='nexus_wrangler', type='group', children=params
        )
        self.tree.setParameters(self.parameters, showTop=False)
        layout.addWidget(self.tree)

        # Connect parameter browse buttons
        self.parameters.child('Calibration').child('poni_file_browse').sigActivated.connect(
            self.browse_poni
        )
        self.parameters.child('NeXus File').child('nexus_file_browse').sigActivated.connect(
            self.browse_nexus
        )
        self.parameters.child('Signal').child('mask_file_browse').sigActivated.connect(
            self.browse_mask
        )
        self.parameters.child('Output').child('h5_dir_browse').sigActivated.connect(
            self.browse_h5_dir
        )

        # Setup thread
        self.thread = nexusThread(
            self.command_queue,
            self.sphere_args,
            self.file_lock,
            self.fname,
            self.nexus_file,
            self.poni,
            self.mask_file,
            self.gi,
            self.th_mtr,
            self.sample_orientation,
            self.tilt_angle,
            self.gi_mode_1d,
            self.gi_mode_2d,
            self.command,
            self.sphere,
            self.data_1d,
            self.data_2d,
            entry=self.entry,
            parent=self,
        )
        self.thread.showLabel.connect(self.statusLabel.setText)
        self.thread.sigUpdateFile.connect(self.sigUpdateFile.emit)
        self.thread.finished.connect(self.finished.emit)
        self.thread.sigUpdate.connect(self.sigUpdateData.emit)
        self.thread.sigUpdateGI.connect(self.sigUpdateGI.emit)

        self._restore_from_session()

    # ── Session persistence ──────────────────────────────────────────

    _SESSION_PARAMS = [
        ('poni_file',           ('Calibration', 'poni_file'),    True,  'poni_file'),
        ('nexus_file',          ('NeXus File', 'nexus_file'),    True,  'nexus_file'),
        ('entry',               ('NeXus File', 'entry'),         False, 'entry'),
        ('mask_file',           ('Signal', 'mask_file'),         True,  'mask_file'),
        ('h5_dir',              ('Output', 'h5_dir'),            True,  'h5_dir'),
        ('gi',                  ('GI', 'Grazing'),               False, 'gi'),
        ('sample_orientation',  ('GI', 'sample_orientation'),    False, 'sample_orientation'),
        ('tilt_angle',          ('GI', 'tilt_angle'),            False, 'tilt_angle'),
        ('gi_mode_1d',          ('GI', 'gi_mode_1d'),            False, 'gi_mode_1d'),
        ('gi_mode_2d',          ('GI', 'gi_mode_2d'),            False, 'gi_mode_2d'),
    ]

    def _restore_from_session(self):
        """Restore parameters from session.json."""
        data = load_session()
        if not data:
            return
        for skey, param_path, is_path, attr in self._SESSION_PARAMS:
            val = data.get(skey)
            if val is None:
                continue
            if is_path and not os.path.exists(val):
                continue
            try:
                p = self.parameters
                for name in param_path:
                    p = p.child(name)
                p.setValue(val)
                if attr:
                    setattr(self, attr, val)
            except Exception:
                pass

        # Restore PONI
        poni_file = self.parameters.child('Calibration').child('poni_file').value()
        if poni_file and os.path.exists(poni_file):
            self.poni = PONI.from_poni_file(poni_file)

    def _save_to_session(self):
        """Save parameters to session.json."""
        data = load_session() or {}
        for skey, param_path, _is_path, _attr in self._SESSION_PARAMS:
            try:
                p = self.parameters
                for name in param_path:
                    p = p.child(name)
                data[skey] = p.value()
            except Exception:
                pass
        save_session(data)

    # ── Browse dialogs ───────────────────────────────────────────────

    def browse_poni(self):
        poni_file, _ = QFileDialog.getOpenFileName(
            self, 'Select PONI file', '', 'PONI files (*.poni);;All files (*)')
        if poni_file:
            self.parameters.child('Calibration').child('poni_file').setValue(poni_file)
            self.poni_file = poni_file
            self.poni = PONI.from_poni_file(poni_file)
            self._save_to_session()

    def browse_nexus(self):
        nexus_file, _ = QFileDialog.getOpenFileName(
            self, 'Select NeXus/HDF5 file',
            os.path.dirname(self.nexus_file) if self.nexus_file else '',
            'HDF5 files (*.h5 *.hdf5 *.nxs);;All files (*)')
        if nexus_file:
            self.parameters.child('NeXus File').child('nexus_file').setValue(nexus_file)
            self.nexus_file = nexus_file
            self._save_to_session()

    def browse_mask(self):
        mask_file, _ = QFileDialog.getOpenFileName(
            self, 'Select mask file', '', 'Image files (*.tif *.edf *.npy);;All files (*)')
        if mask_file:
            self.parameters.child('Signal').child('mask_file').setValue(mask_file)
            self.mask_file = mask_file
            self._save_to_session()

    def browse_h5_dir(self):
        h5_dir = QFileDialog.getExistingDirectory(
            self, 'Select output directory',
            self.h5_dir, QFileDialog.ShowDirsOnly)
        if h5_dir:
            self.parameters.child('Output').child('h5_dir').setValue(h5_dir)
            self.h5_dir = h5_dir
            self._save_to_session()

    # ── Thread control ───────────────────────────────────────────────

    def setup(self):
        """Sync parameters to thread before starting."""
        self.nexus_file = self.parameters.child('NeXus File').child('nexus_file').value()
        self.entry = self.parameters.child('NeXus File').child('entry').value()
        self.poni_file = self.parameters.child('Calibration').child('poni_file').value()
        self.mask_file = self.parameters.child('Signal').child('mask_file').value()

        # Load PONI if needed
        if self.poni_file and os.path.exists(self.poni_file):
            self.poni = PONI.from_poni_file(self.poni_file)

        # Output directory
        h5_dir = self.parameters.child('Output').child('h5_dir').value()
        if h5_dir:
            self.h5_dir = h5_dir

        # HDF5 output file
        scan_name = Path(self.nexus_file).stem if self.nexus_file else 'nexus_scan'
        self.fname = os.path.join(self.h5_dir, f'{scan_name}.nxs')
        self.scan_name = scan_name

        # GI parameters
        self.gi = self.parameters.child('GI').child('Grazing').value()
        self.th_mtr = self.parameters.child('GI').child('th_motor').value()
        self.sample_orientation = self.parameters.child('GI').child('sample_orientation').value()
        self.tilt_angle = self.parameters.child('GI').child('tilt_angle').value()
        # GI modes are driven by the integrator panel (axis1D / axis2D)
        self.gi_mode_1d = self.sphere.bai_1d_args.get('gi_mode_1d', 'q_total')
        self.gi_mode_2d = self.sphere.bai_2d_args.get('gi_mode_2d', 'qip_qoop')

        # Recreate thread with current params
        self.thread = nexusThread(
            self.command_queue,
            self.sphere_args,
            self.file_lock,
            self.fname,
            self.nexus_file,
            self.poni,
            self.mask_file,
            self.gi,
            self.th_mtr,
            self.sample_orientation,
            self.tilt_angle,
            self.gi_mode_1d,
            self.gi_mode_2d,
            self.command,
            self.sphere,
            self.data_1d,
            self.data_2d,
            entry=self.entry,
            parent=self,
        )
        self.thread.showLabel.connect(self.statusLabel.setText)
        self.thread.sigUpdateFile.connect(self.sigUpdateFile.emit)
        self.thread.finished.connect(self.finished.emit)
        self.thread.sigUpdate.connect(self.sigUpdateData.emit)
        self.thread.sigUpdateGI.connect(self.sigUpdateGI.emit)
        self.sigUpdateGI.emit(self.gi)

        self.thread.file_lock = self.file_lock
        self.thread.sphere_args = self.sphere_args
        self.thread.sphere = self.sphere
        self.thread.data_1d = self.data_1d
        self.thread.data_2d = self.data_2d
        self.thread.command = self.command

    def start(self):
        self.command = 'start'
        self.thread.command = 'start'
        self.startButton.setEnabled(False)
        self.pauseButton.setEnabled(True)
        self.stopButton.setEnabled(True)
        self.continueButton.setEnabled(False)
        self._save_to_session()
        self.sigStart.emit()

    def pause(self):
        self.command = 'pause'
        self.thread.command = 'pause'
        self.pauseButton.setEnabled(False)
        self.continueButton.setEnabled(True)

    def cont(self):
        self.command = 'continue'
        self.thread.command = 'continue'
        self.pauseButton.setEnabled(True)
        self.continueButton.setEnabled(False)

    def stop(self):
        self.command = 'stop'
        self.thread.command = 'stop'
        self.startButton.setEnabled(True)
        self.pauseButton.setEnabled(False)
        self.continueButton.setEnabled(False)
        self.stopButton.setEnabled(False)

    def enabled(self, enable):
        self.startButton.setEnabled(enable)
        self.tree.setEnabled(enable)



# -*- coding: utf-8 -*-
"""
@author: walroth
"""
# Standard library imports
import sys
# Qt imports
import pyqtgraph as pg
from pyqtgraph import QtGui
from PySide6 import QtWidgets

# This module imports


class RangeSliderItem(pg.GraphicsWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.layout = QtGui.QGraphicsGridLayout()
        self.setLayout(self.layout)
        self.gradient = pg.GradientEditorItem(orientation="right")
        self.axis = pg.AxisItem(orientation="left")
        self.layout.addItem(self.gradient, 0, 1)
        self.layout.addItem(self.axis, 0, 0)


class RangeSliderWidget(pg.GraphicsView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.item = RangeSliderItem()
        self.setCentralItem(self.item)


class Test(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.vlayout = QtWidgets.QHBoxLayout()
        self.setLayout(self.vlayout)
        self.gradiant = RangeSliderWidget()
        self.vlayout.addWidget(self.gradiant)
        self.show()

if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    test = Test()
    test.show()
    print("at exec")
    app.exec()




# -*- coding: utf-8 -*-
"""
@author: walroth
"""

# Standard library imports
from queue import Queue
import threading
import traceback

# Other imports

# Qt imports
from pyqtgraph import Qt
from pyqtgraph.parametertree import Parameter

# This module imports
from xdart.modules.ewald import EwaldSphere


class wranglerWidget(Qt.QtWidgets.QWidget):
    """Base class for wranglers. Extending this ensures all methods,
    signals, and attributes expected by ttheta_widget are present.
    Threads should be started by use of sigStart.emit, which ensures
    tthetaWidget handles initiation.
    
    attributes:
        command_queue: Queue, used to send commands to thread
        file_lock, mp.Condition, process safe lock for file access
        fname: str, path to data file
        parameters: pyqtgraph Parameter, stores parameters from user
        scan_name: str, current scan name, used to handle syncing data
        sphere_args: dict, used as **kwargs in sphere initialization.
            see EwaldSphere.
        thread: wranglerThread or subclass, QThread for controlling
            processes
    
    methods:
        enabled: Enables or disables interactivity
        set_fname: Method to safely change file name
        setup: Syncs thread parameters prior to starting
    
    signals:
        finished: Should be connected to thread.finished signal
        sigStart: Tells tthetaWidget to start the thread and prepare
            for new data.
        sigUpdateData: int, signals a new arch has been added.
        sigUpdateFile: (str, str, bool, str, bool, bool), sends new scan_name, file name
            GI flag (grazing incidence), theta motor for GI, single_image and
            series_average flag to static_scan_Widget.
    """
    sigStart = Qt.QtCore.Signal()
    sigUpdateData = Qt.QtCore.Signal(int)
    # sigUpdateArch = Qt.QtCore.Signal(dict)
    sigUpdateFile = Qt.QtCore.Signal(str, str, bool, str, bool, bool)
    sigUpdateGI = Qt.QtCore.Signal(bool)
    finished = Qt.QtCore.Signal()
    started = Qt.QtCore.Signal()

    def __init__(self, fname, file_lock, parent=None):
        """fname: str, file path
        file_lock: mp.Condition, process safe lock
        """
        super().__init__(parent)
        self.file_lock = file_lock
        self.fname = fname
        self.scan_name = 'null_thread'
        self.parameters = Parameter.create(
            name='wrangler_widget', type='int', value=0
        )
        self.sphere_args = {}

        self.command_queue = Queue()
        self.thread = wranglerThread(self.command_queue, self.sphere_args, self.fname, self.file_lock, self)
        self.thread.finished.connect(self.finished.emit)
        self.thread.started.connect(self.started.emit)
        self.thread.sigUpdate.connect(self.sigUpdateData.emit)
        # self.thread.sigUpdateArch.connect(self.sigUpdateArch.emit)
        self.thread.sigUpdateGI.connect(self.sigUpdateGI.emit)

    def enabled(self, enable):
        """Use this function to control what is enabled and disabled
        during integration.
        """
        pass

    def setup(self):
        """Sets the thread child object. Called by tthetaWidget prior
        to starting thread.
        """
        # Disconnect old thread signals to avoid duplicate emissions
        try:
            self.thread.finished.disconnect(self.finished.emit)
            self.thread.started.disconnect(self.started.emit)
            self.thread.sigUpdate.disconnect(self.sigUpdateData.emit)
            self.thread.sigUpdateGI.disconnect(self.sigUpdateGI.emit)
        except (TypeError, RuntimeError):
            pass  # Signals were never connected or already disconnected
        self.thread = wranglerThread(self.command_queue, self.sphere_args, self.fname, self.file_lock, self)
        self.thread.finished.connect(self.finished.emit)
        self.thread.started.connect(self.started.emit)
        self.thread.sigUpdate.connect(self.sigUpdateData.emit)
        self.thread.sigUpdateGI.connect(self.sigUpdateGI.emit)

    def set_fname(self, fname):
        """Changes fname attribute of self and thread.
        args:
            fname: str, path for new file.
        """
        with self.file_lock:
            if not self.thread.isRunning():
                self.fname = fname
                self.thread.fname = fname


class wranglerThread(Qt.QtCore.QThread):
    """Base class for wranglerThreads. Used to manage processes
    including data and command queues. Subclasses should override the
    run method.
    
    attributes:
        command_q: mp.Queue, queue to send commands to process
        file_lock: mp.Condition, process safe lock for file access
        fname: str, path to data file.
        input_q: mp.Queue, queue for commands sent from parent
        signal_q: mp.Queue, queue for commands sent from process
        sphere_args: dict, used as **kwargs in sphere initialization.
            see EwaldSphere.
    
    methods:
        run: Called by start, main thread task.
    
    signals:
        sigUpdate: int, signals a new arch has been added.
        sigUpdateFile: (str, str, bool, str, bool, bool), sends new scan_name, file name
            GI flag (grazing incidence), theta motor for GI, single_image and
            series_average flag to static_scan_Widget.
        sigUpdateGI: bool, signals the grazing incidence condition has changed.
    """
    sigUpdate = Qt.QtCore.Signal(int)
    # sigUpdateArch = Qt.QtCore.Signal(dict)
    sigUpdateFile = Qt.QtCore.Signal(str, str, bool, str, bool, bool)
    sigUpdateGI = Qt.QtCore.Signal(bool)

    def __init__(self, command_queue, sphere_args, fname, file_lock,
                 parent=None):
        """command_queue: mp.Queue, queue for commands sent from parent
        sphere_args: dict, used as **kwargs in sphere initialization.
            see EwaldSphere.
        fname: str, path to data file.
        file_lock: mp.Condition, process safe lock for file access
        """
        super().__init__(parent)
        self.input_q = command_queue # thread queue
        self.sphere_args = sphere_args
        self.fname = fname
        self.file_lock = file_lock
        self.signal_q = Queue()
        self.command_q = Queue()
    
    def run(self):
        """Main task. Subclasses (e.g. specThread) override this."""
        pass



# -*- coding: utf-8 -*-
"""
@author: walroth
"""
# Standard library imports
import logging
import os
import time
import gc

logger = logging.getLogger(__name__)

# This module imports
import re
import numpy as np

from ssrl_xrd_tools.core.containers import IntegrationResult1D, IntegrationResult2D
from ssrl_xrd_tools.io.export import read_xye
from ssrl_xrd_tools.io.image import read_image, count_frames
from xdart.utils.session import load_session, save_session
from .ui.h5viewerUI import Ui_Form
from xdart.modules.ewald import EwaldArch
from .sphere_threads import fileHandlerThread
from ...widgets import defaultWidget
from xdart import utils
from xdart.utils import catch_h5py_file as catch
from xdart.utils.h5pool import get_pool

# Qt imports
from pyqtgraph import Qt
from pyqtgraph.Qt import QtWidgets, QtCore, QtGui


QTreeWidget = QtWidgets.QTreeWidget
QTreeWidgetItem = QtWidgets.QTreeWidgetItem
QWidget = QtWidgets.QWidget
QFileDialog = QtWidgets.QFileDialog
QItemSelectionModel = QtCore.QItemSelectionModel


class H5Viewer(QWidget):
    """Widget for displaying the contents of an EwaldSphere object and
    a basic file explorer. Also holds menus for more general tasks like
    setting defaults.
    
    attributes:
        (QAction attributes not shown, associated menus are)
        exportMenu: Sub-menu for exporting images and 1d data
        file_lock: Condition, lock governing file access
        fileMenu: Menu for saving files and exporting data
        fname: Current data file name
        layout: ui layout TODO: this can stay with ui
        paramMenu: Menu for saving and loading defaults
        toolbar: QToolBar, holds the menus
        ui: Ui_Form from qtdesigner

    methods:
        set_data: Sets the data in the dataList
        set_open_enabled: Sets the ability to open scans to enabled or
            disables
        update: Updates files in scansList
        TODO: Rename the methods and attributes based on what they
            actually do
    """
    sigNewFile = Qt.QtCore.Signal(str)
    sigUpdate = Qt.QtCore.Signal()
    sigThreadFinished = Qt.QtCore.Signal()

    def __init__(self, file_lock, local_path, dirname,
                 sphere, arch, arch_ids, arches,
                 data_1d, data_2d,
                 parent=None):
        super().__init__(parent)
        self.local_path = local_path
        self.file_lock = file_lock
        self.dirname = dirname

        # Data Objects
        self.sphere = sphere
        self.arch = arch
        self.arch_ids = arch_ids
        self.arches = arches
        self.data_1d = data_1d
        self.data_2d = data_2d
        self.new_scan = True
        self.update_2d = True
        self.auto_last = True
        self.latest_idx = None
        self.new_scan_loaded = False
        # Viewer mode: None (normal), 'image', or 'xye'
        self.viewer_mode = None

        # Link UI
        self.ui = Ui_Form()
        self.ui.setupUi(self)

        # self.layout = self.ui.layout
        self.layout = self.ui.gridLayout
        self.defaultWidget = defaultWidget()
        self.defaultWidget.sigSetUserDefaults.connect(self.set_user_defaults)

        # Toolbar setup
        self.toolbar = QtWidgets.QToolBar('Tools')

        # File menu setup (part of toolbar)
        self.actionOpenFolder = QtGui.QAction()
        self.actionOpenFolder.setText('Open Folder')

        self.actionSetDefaults = QtGui.QAction()
        self.actionSetDefaults.setText('Advanced...')

        self.actionSaveDataAs = QtGui.QAction()
        self.actionSaveDataAs.setText('Save As')

        self.actionNewFile = QtGui.QAction()
        self.actionNewFile.setText('New')

        self.exportMenu = QtWidgets.QMenu()
        self.exportMenu.setTitle('Export')

        self.actionSaveImage = QtGui.QAction()
        self.actionSaveImage.setText('Current Image')
        self.exportMenu.addAction(self.actionSaveImage)

        self.actionSaveArray = QtGui.QAction()
        self.actionSaveArray.setText('Current 1D Array')
        self.exportMenu.addAction(self.actionSaveArray)
        
        self.paramMenu = QtWidgets.QMenu()
        self.paramMenu.setTitle('Config')
        
        self.actionSaveParams = QtGui.QAction()
        self.actionSaveParams.setText('Save')
        self.actionSaveParams.triggered.connect(self.defaultWidget.save_defaults)
        self.paramMenu.addAction(self.actionSaveParams)
        
        self.actionLoadParams = QtGui.QAction()
        self.actionLoadParams.setText('Load')
        self.actionLoadParams.triggered.connect(self.defaultWidget.load_defaults)
        self.paramMenu.addAction(self.actionLoadParams)
        
        self.paramMenu.addAction(self.actionSetDefaults)

        self.fileMenu = QtWidgets.QMenu()
        self.fileMenu.addAction(self.actionOpenFolder)
        self.fileMenu.addAction(self.actionNewFile)
        self.fileMenu.addAction(self.actionSaveDataAs)
        self.fileMenu.addMenu(self.exportMenu)

        self.fileButton = QtWidgets.QToolButton()
        self.fileButton.setText('File')
        self.fileButton.setPopupMode(QtWidgets.QToolButton.InstantPopup)
        self.fileButton.setMenu(self.fileMenu)
        
        self.paramButton = QtWidgets.QToolButton()
        self.paramButton.setText('Config')
        self.paramButton.setPopupMode(QtWidgets.QToolButton.InstantPopup)
        self.paramButton.setMenu(self.paramMenu)
        # End file menu setup

        self.toolbar.addWidget(self.fileButton)
        self.toolbar.addWidget(self.paramButton)
        # End toolbar setup

        self.layout.addWidget(self.toolbar, 0, 0, 1, 2)
        self.actionSetDefaults.triggered.connect(self.defaultWidget.show)
        
        self.ui.listScans.itemDoubleClicked.connect(self.scans_clicked)
        self.ui.listScans.itemClicked.connect(self._scans_single_clicked)
        self.ui.listScans.currentItemChanged.connect(self._scans_current_changed)
        self.ui.listScans.itemSelectionChanged.connect(self._scans_selection_changed)
        self.ui.listScans.installEventFilter(self)
        self.ui.listData.itemSelectionChanged.connect(self.data_changed)
        self.ui.show_all.clicked.connect(self.show_all)
        self.actionOpenFolder.triggered.connect(self.open_folder)
        self.actionSaveDataAs.triggered.connect(self.save_data_as)
        self.actionNewFile.triggered.connect(self.new_file)
        
        self.file_thread = fileHandlerThread(self.sphere, self.arch,
                                             self.file_lock,
                                             arch_ids=self.arch_ids,
                                             arches=self.arches,
                                             data_1d=self.data_1d,
                                             data_2d=self.data_2d)
        self.file_thread.sigTaskDone.connect(self.thread_finished)
        self.file_thread.sigNewFile.connect(self.sigNewFile.emit)
        self.file_thread.sigUpdate.connect(self.sigUpdate.emit)
        self.file_thread.start(Qt.QtCore.QThread.LowPriority)

        self._h5pool = get_pool()
        
    def load_starting_defaults(self):
        default_path = os.path.join(self.local_path, "last_defaults.json")
        if os.path.exists(default_path):
            self.defaultWidget.load_defaults(fname=default_path)
        else:
            self.defaultWidget.save_defaults(fname=default_path)

    def set_user_defaults(self):
        default_path = os.path.join(self.local_path, "last_defaults.json")
        self.defaultWidget.save_defaults(fname=default_path)

    def update(self):
        """Calls both update_scans and update_data.
        """
        # self.update_scans()
        self.update_data()

        # Restore session
        session = load_session()
        saved_dir = session.get('data_dir', '')
        if saved_dir and os.path.isdir(saved_dir):
            self.dirname = saved_dir
            self.update_scans()

    # File extensions for viewer modes
    _IMAGE_EXTS = {'.tif', '.tiff', '.raw', '.edf', '.h5', '.hdf5', '.nxs'}
    _XYE_EXTS = {'.xye'}

    @staticmethod
    def _natural_sort_key(text):
        return [int(c) if c.isdigit() else c.lower()
                for c in re.split(r'(\d+)', text)]

    def update_scans(self):
        """Populate listScans with files in the current directory.

        In normal mode, shows HDF5 files and directories.
        In image viewer mode, shows image files and directories.
        In xye viewer mode, shows xye files and directories.
        """
        if not os.path.exists(self.dirname):
            return

        self.ui.listScans.clear()
        self.ui.listScans.addItem('..')

        names = sorted(os.listdir(self.dirname), key=self._natural_sort_key)
        for name in names:
            abspath = os.path.join(self.dirname, name)
            if os.path.isdir(abspath):
                self.ui.listScans.addItem(name + '/')
            else:
                ext = os.path.splitext(name)[1].lower()
                if self.viewer_mode == 'image':
                    if ext in self._IMAGE_EXTS:
                        self.ui.listScans.addItem(name)
                elif self.viewer_mode == 'xye':
                    if ext in self._XYE_EXTS:
                        self.ui.listScans.addItem(name)
                else:
                    # Normal mode: only HDF5/NeXus scan files
                    if name.split('.')[-1] in ('h5', 'hdf5', 'nxs'):
                        self.ui.listScans.addItem(name)
    
    def update_data(self):
        """Updates list with all arch ids.
        """
        if self.sphere.name == "null_main":
            return

        # with self.sphere.sphere_lock:
        _idxs = [str(i) for i in list(self.sphere.arches.index)]

        if len(_idxs) == 0:
            self.ui.listData.clear()
            # self.ui.listData.addItem('No Data')
            return

        lw = self.ui.listData
        items = [lw.item(x).text() for x in range(lw.count())]
        eq = _idxs == items

        if (len(_idxs) > 1) and (_idxs == items):
            if self.new_scan_loaded:
                self.new_scan_loaded = False
                self.ui.listData.setCurrentRow(-1)
                self.arch_ids = []
                return
            if self.auto_last and isinstance(self.latest_idx, int) and str(self.latest_idx) in _idxs:
                self.ui.listData.blockSignals(True)
                matched = self.ui.listData.findItems(str(self.latest_idx), QtCore.Qt.MatchExactly)
                for item in matched:
                    self.ui.listData.setCurrentItem(item)
                self.ui.listData.blockSignals(False)
                self.data_changed()
            return
        if (len(_idxs) > 1) and (len(_idxs) == len(items)):
            if self.auto_last and isinstance(self.latest_idx, int):
                self.ui.listData.blockSignals(True)
                matched = self.ui.listData.findItems(str(self.latest_idx), QtCore.Qt.MatchExactly)
                for item in matched:
                    self.ui.listData.setCurrentItem(item)
                self.ui.listData.blockSignals(False)
                self.data_changed()
            return

        previous_loc = self.ui.listData.currentRow()
        previous_sel = [item.text() for item in self.ui.listData.selectedItems()]

        # Block signals while rebuilding the list to prevent spurious
        # itemSelectionChanged → data_changed → sigUpdate cascades.
        self.ui.listData.blockSignals(True)

        self.ui.listData.clear()
        self.ui.listData.insertItems(0, _idxs)

        if self.new_scan_loaded:
            self.new_scan_loaded = False
            self.ui.listData.setCurrentRow(-1)
            self.arch_ids.clear()
            self.ui.listData.blockSignals(False)
            return

        if self.auto_last and isinstance(self.latest_idx, int) and (str(self.latest_idx) in _idxs):
            items = self.ui.listData.findItems(str(self.latest_idx), QtCore.Qt.MatchExactly)
            if len(items):
                for item in items:
                    self.ui.listData.setCurrentItem(item)
            self.ui.listData.blockSignals(False)
            self.data_changed()
            return

        if previous_loc > self.ui.listData.count() - 1:
            previous_loc = self.ui.listData.count() - 1

        if len(previous_sel) < 2:
            self.ui.listData.setCurrentRow(previous_loc)
        else:
            for text in previous_sel:
                matched = self.ui.listData.findItems(text, QtCore.Qt.MatchExactly)
                for item in matched:
                    item.setSelected(True)

        self.ui.listData.blockSignals(False)
        self.data_changed()

    def show_all(self):

        if len(self.sphere.arches.index) > 0:
            self.arch_ids.clear()
            self.arch_ids += self.sphere.arches.index

        self.new_scan = False
        self.data_changed(show_all=True)

    def thread_finished(self, task):
        if task != "load_arch":
            self.update()
            if getattr(self, '_auto_select_last_on_finish', False):
                self._auto_select_last_on_finish = False
                if self.ui.listData.count() > 0:
                    self.ui.listData.setCurrentRow(self.ui.listData.count() - 1)
        self.sigThreadFinished.emit()

        gc.collect()
    
    def _scans_single_clicked(self, q):
        """Handle single click in listScans — only acts in viewer mode.

        XYE mode uses _scans_selection_changed instead to avoid double-firing.
        """
        if self.viewer_mode is not None and self.viewer_mode != 'xye':
            self.scans_clicked(q)

    def _scans_current_changed(self, current, previous):
        """Handle arrow-key navigation in listScans (image viewer only).

        Loads files on selection change so the user can browse with
        arrow keys.  Directories are NOT auto-entered — use click or
        Enter for navigation.

        XYE mode uses _scans_selection_changed instead (fires after
        the selection is fully updated, avoiding off-by-one with
        Shift+arrow).
        """
        if current is None or self.viewer_mode is None:
            return
        # XYE mode: handled by _scans_selection_changed
        if self.viewer_mode == 'xye':
            return
        item_text = current.text()
        # Skip directories and ".." — don't auto-navigate on arrow keys
        if item_text == '..' or item_text.endswith('/'):
            return
        self.scans_clicked(current)

    def _scans_selection_changed(self):
        """Handle selection changes in listScans (XYE viewer mode only).

        Uses itemSelectionChanged which fires after the selection is
        fully updated, so Shift+arrow works correctly for multi-select.
        """
        if self.viewer_mode != 'xye':
            return
        selected = self.ui.listScans.selectedItems()
        if not selected:
            return
        # Skip if only directories/".." are selected
        has_files = any(
            not item.text().endswith('/') and item.text() != '..'
            for item in selected
        )
        if has_files:
            self._load_xye_files()

    def eventFilter(self, obj, event):
        """Handle Enter/Return key on listScans to navigate into folders."""
        if obj is self.ui.listScans and event.type() == event.Type.KeyPress:
            from PySide6.QtCore import Qt
            if event.key() in (Qt.Key.Key_Return, Qt.Key.Key_Enter):
                current = self.ui.listScans.currentItem()
                if current is not None:
                    item_text = current.text()
                    if item_text == '..' or item_text.endswith('/'):
                        self.scans_clicked(current)
                        return True
        return super().eventFilter(obj, event)

    def scans_clicked(self, q):
        """Handles items being clicked/double-clicked in listScans.

        Normal mode: navigates folders or loads sphere data from HDF5.
        Image Viewer: loads a single image or populates listData for multi-frame files.
        XYE Viewer: loads a single xye file as a 1D line.
        """
        try:
            item_text = q.data(0)

            # Navigation: ".." or folder
            if item_text == '..':
                if self.dirname[-1] in ['/', '\\']:
                    up = os.path.dirname(self.dirname[:-1])
                else:
                    up = os.path.dirname(self.dirname)
                if os.path.isdir(up) and os.path.splitdrive(up)[1] != '':
                    self.dirname = up
                    self.update_scans()
                return
            if '/' in item_text:
                dirname = os.path.join(self.dirname, item_text)
                if os.path.isdir(dirname):
                    self.dirname = dirname
                    self.update_scans()
                return

            if item_text == 'No scans':
                return

            fpath = os.path.join(self.dirname, item_text)

            # ── Viewer modes ──────────────────────────────────────────
            if self.viewer_mode == 'xye':
                self._load_xye_files()
                return
            if self.viewer_mode == 'image':
                self._load_image_file(fpath)
                return

            # ── Normal mode: open HDF5 scan ───────────────────────────
            self.set_file(fpath)
            self.new_scan_loaded = True
        except AttributeError:
            pass

    # ── Viewer mode loaders ───────────────────────────────────────────────

    def _load_xye_files(self):
        """Load all selected xye files from listScans for overlay.

        Each file gets a sequential index (1, 2, 3, …) in data_1d.
        listData is populated with filenames and all rows are selected
        so the display frame renders every curve.
        """
        selected = self.ui.listScans.selectedItems()
        if not selected:
            return

        self.data_1d.clear()
        self.data_2d.clear()
        self.arch_ids.clear()

        idx = 1
        for item in selected:
            item_text = item.text()
            # Skip directories
            if item_text == '..' or item_text.endswith('/'):
                continue
            fpath = os.path.join(self.dirname, item_text)
            try:
                xdata, ydata, sigma = read_xye(fpath)
            except Exception:
                logger.debug("Could not load xye file %s", fpath, exc_info=True)
                continue

            # Guess unit from filename prefix
            fname_lower = os.path.basename(fpath).lower()
            unit = 'q_A^-1' if fname_lower.startswith('iq') else '2th_deg'

            int_1d = IntegrationResult1D(
                radial=xdata, intensity=ydata, sigma=sigma, unit=unit,
            )
            arch = EwaldArch(idx=idx, static=True, gi=False)
            arch.int_1d = int_1d
            arch.scan_info = {'source_file': os.path.basename(fpath)}

            self.data_1d[idx] = arch
            self.arch_ids.append(str(idx))
            idx += 1

        if len(self.data_1d) == 0:
            return

        # Populate listData with loaded filenames (all selected).
        # Display filename but store numeric index in UserRole so
        # data_changed can map back to data_1d keys.
        self.ui.listData.blockSignals(True)
        self.ui.listData.clear()
        for key in self.data_1d:
            arch = self.data_1d[key]
            fname = arch.scan_info.get('source_file', f'file_{key}')
            display_name = os.path.basename(fname)
            item = QtWidgets.QListWidgetItem(display_name)
            item.setData(QtCore.Qt.UserRole, key)
            self.ui.listData.addItem(item)
        self.ui.listData.selectAll()
        self.ui.listData.blockSignals(False)

        self.sigUpdate.emit()

    def _load_image_file(self, fpath):
        """Load an image file for viewing.

        For multi-frame files (Eiger HDF5 masters, tiff stacks),
        populate listData with frame indices.  For single-frame
        files, display the image directly.
        """
        self.data_1d.clear()
        self.data_2d.clear()
        self.arch_ids.clear()
        self.ui.listData.clear()

        ext = os.path.splitext(fpath)[1].lower()
        nframes = 1

        # Check for multi-frame files
        if ext in ('.h5', '.hdf5', '.nxs'):
            nframes = count_frames(fpath)
            if nframes == 0:
                # count_frames failed — try loading as single frame
                nframes = 1
        elif ext in ('.tif', '.tiff'):
            try:
                import fabio
                img = fabio.open(fpath)
                nframes = img.nframes
                img.close()
            except Exception:
                logger.debug("Failed to detect frame count from TIFF file %s", fpath, exc_info=True)
                nframes = 1

        # HDF5/NeXus files always show frame numbers (even with 1 frame)
        is_hdf5 = ext in ('.h5', '.hdf5', '.nxs')

        if nframes > 1 or (is_hdf5 and nframes >= 1):
            # Multi-frame or HDF5: populate listData with frame numbers
            for i in range(nframes):
                self.ui.listData.addItem(str(i + 1))
            # Store the file path so data_changed can load individual frames
            self._viewer_image_path = fpath
            self._viewer_image_nframes = nframes
            # Load and display first frame
            self._load_single_frame(fpath, frame_idx=0, arch_idx=1)
            self.arch_ids.append('1')
            self.ui.listData.setCurrentRow(0)
        else:
            # Single frame (tif, raw, edf): load directly, leave listData blank
            self._viewer_image_path = None
            self._load_single_frame(fpath, frame_idx=0, arch_idx=1)
            self.arch_ids.append('1')

        self.sigUpdate.emit()

    # Common detector shapes to try for raw binary files (name, shape)
    _RAW_DETECTOR_FALLBACKS = [
        ('Pilatus 100k', (195, 487)),
        ('Pilatus 300k', (619, 487)),
        ('Pilatus 300kw', (195, 1475)),
        ('Pilatus 1M', (1043, 981)),
        ('Rayonix MX225', (3072, 3072)),
        ('Rayonix SX165', (2048, 2048)),
    ]

    def _load_single_frame(self, fpath, frame_idx=0, arch_idx=1):
        """Load a single frame from an image file into data_2d."""
        try:
            img_data = np.asarray(
                read_image(fpath, frame=frame_idx), dtype=float,
            )
        except Exception:
            logger.debug("Failed to load frame %d from %s", frame_idx, fpath, exc_info=True)
            # For raw files, try common detector shapes
            ext = os.path.splitext(fpath)[1].lower()
            if ext == '.raw':
                img_data = self._try_raw_detectors(fpath)
                if img_data is None:
                    logger.warning('Cannot load %s — raw file does not match any '
                                   'known detector shape.', os.path.basename(fpath))
                    return
            else:
                logger.warning('Could not load image %s frame %d', fpath, frame_idx)
                return

        self.data_2d[int(arch_idx)] = {
            'map_raw': img_data,
            'bg_raw': np.zeros_like(img_data),
            'mask': None,
            'int_2d': None,
            'gi_2d': {},
            'thumbnail': None,
        }
        # Minimal data_1d entry so display doesn't crash
        arch = EwaldArch(idx=arch_idx, static=True, gi=False)
        arch.scan_info = {'source_file': os.path.basename(fpath)}
        self.data_1d[int(arch_idx)] = arch
    
    def _try_raw_detectors(self, fpath):
        """Try reading a raw binary file with common detector shapes."""
        for name, shape in self._RAW_DETECTOR_FALLBACKS:
            try:
                img_data = np.asarray(
                    read_image(fpath, detector_shape=shape), dtype=float,
                )
                logger.debug('Loaded %s as %s (%dx%d)',
                             os.path.basename(fpath), name, shape[0], shape[1])
                return img_data
            except Exception:
                logger.debug("Failed to load %s as %s detector shape %s", fpath, name, shape, exc_info=True)
                continue
        return None

    def set_file(self, fname):
        """Changes the data file.

        args:
            fname: str, absolute path for data file
        """
        if fname != '':
            try:
                # with self.file_lock:
                #     with catch_h5py_file(fname, 'a') as _:
                #         pass

                self.ui.listData.itemSelectionChanged.disconnect(self.data_changed)
                self.ui.listData.clear()
                self.ui.listData.addItem('Loading...')
                # self.set_open_enabled(False)
                self.file_thread.fname = fname
                self.file_thread.queue.put("set_datafile")
                self.ui.listData.itemSelectionChanged.connect(self.data_changed)
                self.new_scan = True
            except Exception:
                logger.exception("Failed to set file: %s", fname)
                return

    def data_changed(self, show_all=False):
        """Connected to itemSelectionChanged signal of listData.

        In viewer image mode with a multi-frame file, loads the
        selected frame on demand.  Otherwise falls through to the
        normal HDF5-based loading.
        """
        if not show_all:
            self.arch_ids.clear()
            items = self.ui.listData.selectedItems()
            if self.viewer_mode == 'xye':
                # XYE viewer stores the int key in UserRole
                self.arch_ids += sorted(
                    [str(item.data(QtCore.Qt.UserRole)) for item in items
                     if item.data(QtCore.Qt.UserRole) is not None])
            else:
                self.arch_ids += sorted([str(item.text()) for item in items])
            idxs = self.arch_ids
        else:
            idxs = self.arch_ids

        if (len(idxs) == 0) or ('No data' in idxs):
            time.sleep(0.1)
            return

        # ── Image viewer ─────────────────────────────────────────────
        if self.viewer_mode == 'image':
            viewer_path = getattr(self, '_viewer_image_path', None)
            if viewer_path is not None:
                # Multi-frame: load selected frames on demand
                for idx_str in idxs:
                    idx = int(idx_str)
                    if idx not in self.data_2d:
                        self._load_single_frame(
                            viewer_path,
                            frame_idx=idx - 1,  # listData shows 1-based
                            arch_idx=idx,
                        )
            # Single-frame: data already loaded by _load_image_file
            self.sigUpdate.emit()
            return

        # ── XYE viewer: data already loaded by scans_clicked ─────────
        if self.viewer_mode == 'xye':
            self.sigUpdate.emit()
            return

        # ── Normal mode: load from HDF5 ──────────────────────────────
        load_2d = self.update_2d

        if len(self.sphere.arches.index) > 1:
            if len(idxs) == len(self.sphere.arches.index):
                load_2d = False

        if load_2d:
            idxs_memory = [int(idx) for idx in idxs if int(idx) in self.data_2d.keys()]
        else:
            idxs_memory = [int(idx) for idx in idxs if int(idx) in self.data_1d.keys()]

        if load_2d:
            if (len(self.arches) == 0) or (len(self.data_2d) == 0):
                self.arches.update({'sum_int_2d': None, 'sum_map_raw': 0})
                self.arches.update({'idxs': [], 'add_idxs': [], 'sub_idxs': []})

            if len(idxs) > 1:
                self.get_arches_sum(idxs, idxs_memory)

            self.arches['idxs'] = [int(idx) for idx in idxs]

        arch_ids = [int(idx) for idx in idxs
                    if int(idx) not in idxs_memory]

        if len(arch_ids) > 0:
            self.load_arches_data(arch_ids, load_2d)

        self.sigUpdate.emit()

    gc.collect()

    def closeEvent(self, event):
        self._h5pool.close_all()
        super().closeEvent(event)

    def data_reset(self):
        """Resets data in memory (self.arches, self.arch_ids, self.data_..
        """
        self._h5pool.close(self.sphere.data_file)
        self.arches.clear()
        self.arch_ids.clear()
        self.data_1d.clear()
        self.data_2d.clear()
        self.new_scan = True

    def open_folder(self):
        """Changes the directory being displayed in the file explorer.
        """
        dirname = QFileDialog().getExistingDirectory(
            caption='Choose Directory',
            dir='',
            options=QFileDialog.ShowDirsOnly
        )
        if os.path.exists(dirname):
            self.dirname = dirname
            save_session({'data_dir': dirname})
            self.arches.clear()
            self.data_1d.clear()
            self.data_2d.clear()
            self.new_scan = True
            self.update_scans()
    
    def set_open_enabled(self, enable):
        """Sets the save and open actions to enable
        
        args:
            enable: bool, if True actions are enabled
        """
        self.actionSaveDataAs.setEnabled(enable)
        self.paramMenu.setEnabled(enable)
        self.actionOpenFolder.setEnabled(enable)
        self.actionNewFile.setEnabled(enable)
        # self.ui.listScans.setEnabled(enable)
    
    def save_data_as(self):
        """Saves all data to hdf5 file. Also sets fname to be the
        selected file.
        """
        fname, _ = QFileDialog.getSaveFileName()
        with self.file_thread.lock:
            self.file_thread.new_fname = fname
            self.file_thread.queue.put("save_data_as")
        self.set_file(fname)
    
    def new_file(self):
        """Calls file dialog and sets the file name.
        """
        fname, _ = QFileDialog.getSaveFileName()
        self.set_file(fname)

    def load_arches_data(self, arch_ids, load_2d):
        """Loads per-frame data from NeXus HDF5 (``entry/frames/``).

        args:
            arch_ids: list of frame indices to load
            load_2d: bool — whether to load 2D data and thumbnails
        """
        file = self._h5pool.get(self.sphere.data_file)
        if file is None:
            # File is being written to by the wrangler thread — skip this read.
            return

        if "entry" not in file or "frames" not in file["entry"]:
            return
        frames_grp = file["entry/frames"]

        for idx in arch_ids:
            try:
                arch = EwaldArch(idx=idx, static=True, gi=self.sphere.gi)
                arch.load_from_nexus(frames_grp, load_2d=load_2d)

                if not load_2d:
                    self.data_1d[int(idx)] = arch.copy(include_2d=False)
                else:
                    self.data_1d[int(idx)] = arch.copy(include_2d=False)
                    self.data_2d[int(idx)] = {'map_raw': arch.map_raw,
                                              'bg_raw': arch.bg_raw,
                                              'mask': arch.mask,
                                              'int_2d': arch.int_2d,
                                              'gi_2d': arch.gi_2d,
                                              'thumbnail': arch.thumbnail}

                    if idx in self.arches['add_idxs']:
                        _d = self.data_2d[int(idx)]['int_2d']
                        if self.arches['sum_int_2d'] is None:
                            self.arches['sum_int_2d'] = _d
                        else:
                            try:
                                self.arches['sum_int_2d'] = self.arches['sum_int_2d'] + _d
                            except (ValueError, AttributeError):
                                self.arches['sum_int_2d'] = _d
                        self.arches['sum_map_raw'] += (self.data_2d[int(idx)]['map_raw'] -
                                                       self.data_2d[int(idx)]['bg_raw'])
                    elif idx in self.arches['sub_idxs']:
                        _d = self.data_2d[int(idx)]['int_2d']
                        if self.arches['sum_int_2d'] is not None:
                            try:
                                self.arches['sum_int_2d'] = self.arches['sum_int_2d'] - _d
                            except (ValueError, AttributeError):
                                self.arches['sum_int_2d'] = _d
                        self.arches['sum_map_raw'] -= (self.data_2d[int(idx)]['map_raw'] -
                                                       self.data_2d[int(idx)]['bg_raw'])

            except KeyError:
                pass

    # Removed legacy load_arch_data — all reads now go through
    # EwaldArch.load_from_nexus via load_arches_data above.

    def get_arches_sum(self, idxs, idxs_memory):
        new_idxs = set([int(idx) for idx in idxs])
        old_idxs, data_keys = set(self.arches['idxs']), set(self.data_2d.keys())

        changed_idxs = new_idxs ^ old_idxs
        load_idxs = changed_idxs - data_keys


        add_from_data = (new_idxs - old_idxs) & data_keys
        add_from_h5 = new_idxs - old_idxs - data_keys
        self.arches['add_idxs'] = [int(k) for k in add_from_h5]

        sub_from_data = (old_idxs - new_idxs) & data_keys
        sub_from_h5 = old_idxs - new_idxs - data_keys
        self.arches['sub_idxs'] = [int(k) for k in sub_from_h5]

        for x in load_idxs:
            if int(x) in idxs_memory:
                idxs_memory.remove(int(x))

        for k in add_from_data:
            _d = self.data_2d[int(k)]['int_2d']
            if self.arches['sum_int_2d'] is None:
                self.arches['sum_int_2d'] = _d
                self.arches['sum_map_raw'] = self.data_2d[int(k)]['map_raw']
            else:
                try:
                    self.arches['sum_int_2d'] = self.arches['sum_int_2d'] + _d
                    self.arches['sum_map_raw'] += self.data_2d[int(k)]['map_raw']
                except (ValueError, AttributeError):
                    self.arches['sum_int_2d'] = _d
                    self.arches['sum_map_raw'] = self.data_2d[int(k)]['map_raw']

        for k in sub_from_data:
            _d = self.data_2d[int(k)]['int_2d']
            if self.arches['sum_int_2d'] is None:
                self.arches['sum_int_2d'] = _d
                self.arches['sum_map_raw'] = self.data_2d[int(k)]['map_raw']
            else:
                try:
                    self.arches['sum_int_2d'] = self.arches['sum_int_2d'] - _d
                    self.arches['sum_map_raw'] -= self.data_2d[int(k)]['map_raw']
                except (ValueError, AttributeError):
                    self.arches['sum_int_2d'] = _d
                    self.arches['sum_map_raw'] = self.data_2d[int(k)]['map_raw']



import h5py
import threading
from collections import OrderedDict


# Module-level singleton — import and use this from anywhere that needs
# to coordinate read handles with write operations.
_pool = None


def get_pool():
    """Return the process-wide H5FilePool, creating it on first call."""
    global _pool
    if _pool is None:
        _pool = H5FilePool(max_open=5)
    return _pool


class H5FilePool:
    """Keeps HDF5 files open in read-only mode for fast repeated access.

    Thread-safety: a threading.Lock guards all mutations.  Writers should call
    ``pause(path)`` before opening a file for writing and ``resume(path)``
    after closing it.  While paused, ``get(path)`` will **not** reopen the
    file, returning ``None`` instead.
    """

    def __init__(self, max_open=5):
        self._files = OrderedDict()  # {path_str: h5py.File}
        self._max = max_open
        self._lock = threading.Lock()
        self._paused = set()  # paths that should not be (re)opened

    def get(self, path):
        """Return open read-only file handle, or *None* if the path is paused.

        Opens the file if needed, evicts LRU if at capacity.
        """
        key = str(path)
        with self._lock:
            if key in self._paused:
                return None
            if key in self._files:
                self._files.move_to_end(key)
                f = self._files[key]
                if f.id.valid:
                    return f
                del self._files[key]
            while len(self._files) >= self._max:
                _, old_f = self._files.popitem(last=False)
                try:
                    old_f.close()
                except Exception:
                    pass
            f = h5py.File(key, 'r')
            self._files[key] = f
            return f

    def close(self, path):
        """Close a cached read handle (e.g. before writing)."""
        key = str(path)
        with self._lock:
            if key in self._files:
                try:
                    self._files.pop(key).close()
                except Exception:
                    pass

    def pause(self, path):
        """Close the read handle *and* prevent ``get()`` from reopening it.

        Call this before opening a file for writing.
        """
        key = str(path)
        with self._lock:
            self._paused.add(key)
            if key in self._files:
                try:
                    self._files.pop(key).close()
                except Exception:
                    pass

    def resume(self, path):
        """Allow ``get()`` to open the file again after a write is done."""
        key = str(path)
        with self._lock:
            self._paused.discard(key)

    def close_all(self):
        with self._lock:
            for f in self._files.values():
                try:
                    f.close()
                except Exception:
                    pass
            self._files.clear()



"""Wrappers around pyFAI GUI tools (calib2, drawmask).

Heavy imports (pyFAI.gui, silx.gui) are deferred to function bodies so they
don't run at application startup.  This avoids silx/Qt backend conflicts and
speeds up the initial load.
"""
import os
import logging

import pyFAI.resources
import pyFAI.calibrant
import pyFAI.detectors
import pyFAI.io.image
import fabio

from pyqtgraph import Qt

logging.basicConfig(level=logging.INFO)
logging.captureWarnings(True)
logger = logging.getLogger(__name__)
try:
    import hdf5plugin  # noqa
except ImportError:
    logger.debug("Unable to load hdf5plugin, backtrace:", exc_info=True)

QFileDialog = Qt.QtWidgets.QFileDialog
logger_uncaught = logging.getLogger("pyFAI-calib2.UNCAUGHT")


def pyFAI_calib2_main():
    # Lazy imports — only needed when user opens calibration window
    import silx
    from silx.gui import qt
    from pyFAI.gui.CalibrationWindow import CalibrationWindow
    from pyFAI.gui.CalibrationContext import CalibrationContext
    from pyFAI.app.calib2 import parse_options, setup_model

    # --help must also work without Qt
    options = parse_options()

    if options.debug:
        logging.root.setLevel(logging.DEBUG)

    if options.opengl:
        silx.config.DEFAULT_PLOT_BACKEND = "opengl"

    pyFAI.resources.silx_integration()
    settings = qt.QSettings(qt.QSettings.IniFormat,
                            qt.QSettings.UserScope,
                            "pyfai",
                            "pyfai-calib2",
                            None)

    context = CalibrationContext(settings)
    context.restoreSettings()

    setup_model(context.getCalibrationModel(), options)

    # Inline subclass to avoid top-level import of CalibrationWindow
    class CalibrationWindowXdart(CalibrationWindow):
        def __init__(self, ctx):
            super().__init__(context=ctx)
            self.context = ctx

        def closeEvent(self, event):
            poniFile = self.model().experimentSettingsModel().poniFile()
            if not poniFile.isSynchronized():
                button = qt.QMessageBox.question(
                    self,
                    "calib2",
                    "The PONI file was not saved.\nDo you really want to close the application?",
                    qt.QMessageBox.Cancel | qt.QMessageBox.No | qt.QMessageBox.Yes,
                    qt.QMessageBox.Yes)
                if button != qt.QMessageBox.Yes:
                    event.ignore()
                    return
            event.accept()
            CalibrationContext._releaseSingleton()

    window = CalibrationWindowXdart(context)
    window.setVisible(True)
    window.setAttribute(qt.Qt.WA_DeleteOnClose, True)
    context.saveSettings()


def pyFAI_drawmask_main(window, image, processFile):
    usage = "pyFAI-drawmask file1.edf file2.edf ..."
    description = """
    Draw a mask, i.e. an image containing the list of pixels which are considered invalid
    (no scintillator, module gap, beam stop shadow, ...).
    """

    window.setImageData(image)
    outfile = os.path.splitext(processFile)[0] + "-mask.edf"
    window.setOutputFile(outfile)
    window.outFile = outfile
    print("Your mask-file will be saved into %s" % outfile)


def get_MaskImageWidgetXdart():
    """Factory function — returns the MaskImageWidgetXdart class.

    Call this lazily instead of importing MaskImageWidgetXdart at the top level.
    """
    from pyFAI.app.drawmask import MaskImageWidget

    class MaskImageWidgetXdart(MaskImageWidget):
        """Window application which allows creating a mask manually."""

        def __init__(self):
            super().__init__()
            self.outFile = None

        def closeEvent(self, event):
            if os.path.exists(self.outFile):
                mask_file = os.path.basename(self.outFile)
                out_dialog = Qt.QtWidgets.QMessageBox()
                out_dialog.setText(f'{mask_file} saved in Image directory')
                out_dialog.exec()
            event.accept()

    return MaskImageWidgetXdart



# -*- coding: utf-8 -*-
"""
@author: walroth
"""

# Standard library imports

# Other imports
import pandas as pd

# Qt imports
from pyqtgraph import Qt, QtCore

# This module imports
from ...gui_utils import DFTableModel


class metadataWidget(Qt.QtWidgets.QWidget):
    """Widget for displaying metadata.
    
    attributes:
        layout: QVBoxLayout, widget layout
        tableview: QTableView, viewer for table model
    
    methods:
        update: Updates the data displayed
    """
    def __init__(self, sphere, arch, arch_ids, arches, parent=None):
        super().__init__(parent)
        self.layout = Qt.QtWidgets.QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.layout)
        self.tableview = Qt.QtWidgets.QTableView()
        self.tableview.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.tableview.setModel(DFTableModel())
        self.layout.addWidget(self.tableview)
        self.sphere = sphere
        self.arch = arch
        self.arch_ids = arch_ids
        self.arches = arches

    def update(self):
        """Updates the table with new data.
        """
        #self.tableview.reset()
        if self.arch.idx is None:
            self.tableview.setModel(
                DFTableModel(self.sphere.scan_data.transpose())
            )
        
        else:
            data = pd.DataFrame(self.arch.scan_info, index=[self.arch.idx])
            self.tableview.setModel(DFTableModel(data.transpose()))



from ._widgets import defaultWidget
from .range_widget import rangeWidget
from .vertical_range_slider import RangeSliderWidget
from .image_widget import XDImageWidget, pgImageWidget, pmeshImageWidget
from .mask_widget import MaskWidget



# -*- coding: utf-8 -*-
"""
@author: walroth
"""

# Standard library imports
import traceback

# Other imports
import numpy as np

#Qt Imports
from PySide6 import QtWidgets
from PySide6.QtCore import Signal

import pyqtgraph as pg

from xdart.utils.pgOverrides.ROI import XDEllipseROI, XDPolyLineROI
from xdart.modules.ewald import EwaldArch, EwaldSphere
from pyqtgraph import LineROI, ROI

from .image_widget import XDImageWidget
from .maskWidgetUI import Ui_Form


ADD_PEN = (0, 0, 255)
SUB_PEN = (0, 255, 0)


class MaskWidget(QtWidgets.QWidget):
    newMask = Signal(int, np.ndarray)
    requestMask = Signal(int)

    def __init__(self, parent=None, sphere=None, arch=None):
        super().__init__(parent)
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.image = XDImageWidget(self)
        self.ui.horizontalLayout.insertWidget(0, self.image, 2)
        self.resize(800, 400)

        self.ROIs = []

        self.ui.rectMaskButton.clicked.connect(self.add_rect_roi)
        self.ui.ellipseMaskButton.clicked.connect(self.add_ellipse_roi)
        self.ui.lineMaskButton.clicked.connect(self.add_line_roi)
        self.ui.polyMaskButton.clicked.connect(self.add_poly_roi)

        self.ui.xMinBox.valueChanged.connect(self.update_mask)
        self.ui.xMaxBox.valueChanged.connect(self.update_mask)
        self.ui.yMinBox.valueChanged.connect(self.update_mask)
        self.ui.yMaxBox.valueChanged.connect(self.update_mask)

        self.ui.clearButton.clicked.connect(self.clear_mask)

        self.mask = np.array([])
        self.base_mask = np.array([])
        self.data = np.array([])

        self.ui.archList.currentTextChanged.connect(self._request_data)
        self.ui.setGlobal.clicked.connect(self._send_global)
        self.ui.setCurrent.clicked.connect(self._send_local)

        self.show()

    def set_data(self, arr, base=None):
        self.data = arr.copy()
        if base is None or base.shape != self.data.shape:
            self.base_mask = np.zeros_like(self.data)
        else:
            self.base_mask = base.copy()
        self.ui.xMaxBox.setMaximum(self.data.shape[0])
        self.ui.yMaxBox.setMaximum(self.data.shape[1])

        self.ui.xMinBox.setMaximum(self.data.shape[0])
        self.ui.yMinBox.setMaximum(self.data.shape[1])

        self.ui.xMaxSlider.setMaximum(self.data.shape[0])
        self.ui.yMaxSlider.setMaximum(self.data.shape[1])

        self.ui.xMinSlider.setMaximum(self.data.shape[0])
        self.ui.yMinSlider.setMaximum(self.data.shape[1])

        self.ui.xMaxBox.setValue(self.data.shape[0])
        self.ui.yMaxBox.setValue(self.data.shape[1])

        self.image.raw_image = self.data.copy()
        self.image.update_image()

        self.update_mask()

    def _request_data(self, q=None):
        try:
            out = int(q)
        except ValueError:
            out = -1
        self.requestMask.emit(out)

    def _send_global(self, q=None):
        self._send_and_set(-1)

    def _send_local(self, q=None):
        if self.ui.archList.currentText().lower() == "global":
            self._send_global()
        else:
            try:
                index = int(self.ui.archList.currentText())
                self._send_and_set(index)
            except ValueError:
                print("Invalid index encountered while trying to emit local mask.")

    def _send_and_set(self, index):
        self.newMask.emit(index, self.mask)
        self.base_mask = self.mask.copy()
        self.clear_mask(clear_base=False)

    def add_rect_roi(self):
        x = int(self.data.shape[0]/2)
        y = int(self.data.shape[1]/2)
        w = int(self.data.shape[0]/10)
        h = int(self.data.shape[1]/10)
        pen, sign = self._get_pen_and_sign()
        new = ROI([x,y], [w,h], removable=True, pen=pen)
        new.addScaleHandle([1, 0.5], [0, 0.5])
        new.addScaleHandle([0, 0.5], [1, 0.5])

        new.addScaleHandle([0.5, 0], [0.5, 1])
        new.addScaleHandle([0.5, 1], [0.5, 0])

        new.addScaleHandle([0, 1], [1, 0])
        new.addScaleHandle([1, 0], [0, 1])
        new.addScaleHandle([1, 1], [0, 0])
        new.addScaleHandle([0, 0], [1, 1])
        new.sigRegionChangeFinished.connect(self.update_mask)
        new.sigRemoveRequested.connect(self.remove_roi)

        self._append_roi(new, sign)

    def _get_pen_and_sign(self):
        if self.ui.addROI.isChecked():
            pen = ADD_PEN
            sign = 1
        elif self.ui.subtractROI.isChecked():
            pen = SUB_PEN
            sign = 0
        return pen, sign

    def add_ellipse_roi(self, q=None):
        pen, sign = self._get_pen_and_sign()
        x = int(self.data.shape[0]/2)
        y = int(self.data.shape[1]/2)
        w = int(self.data.shape[0]/10)
        h = int(self.data.shape[1]/10)
        new = XDEllipseROI([x, y], [w, h], removable=True, pen=pen)

        new.sigRegionChangeFinished.connect(self.update_mask)
        new.sigRemoveRequested.connect(self.remove_roi)

        self._append_roi(new, sign)

    def _append_roi(self, new, sign):
        self.ROIs.append((new, sign))
        self.image.imageViewBox.addItem(self.ROIs[-1][0])
        self.update_mask()

    def add_line_roi(self, q=None):
        pen, sign = self._get_pen_and_sign()
        x = int(self.data.shape[0]/2)
        y = int(self.data.shape[1]/2)
        w = int(self.data.shape[0]/10)
        h = int(self.data.shape[1]/10)
        new = LineROI([x, y], [x, y + h], width=w/4, removable=True,
                      pen=pen)

        new.sigRegionChangeFinished.connect(self.update_mask)
        new.sigRemoveRequested.connect(self.remove_roi)

        self._append_roi(new, sign)

    def add_poly_roi(self, q=None):
        pen, sign = self._get_pen_and_sign()
        x = int(self.data.shape[0]/2)
        y = int(self.data.shape[1]/2)
        w = int(self.data.shape[0]/10)
        h = int(self.data.shape[1]/10)
        new = XDPolyLineROI([[x, y], [x, y + h], [x + w, y]], closed=True,
                            removable=True, pen=pen)

        new.sigRegionChangeFinished.connect(self.update_mask)
        new.sigRemoveRequested.connect(self.remove_roi)

        self._append_roi(new, sign)

    def remove_roi(self, q):
        to_remove = []
        for i, roi in enumerate(self.ROIs):
            if roi[0] is q:
                to_remove.append(i)
                self.image.imageViewBox.removeItem(roi[0])
        for i in to_remove:
            del(self.ROIs[i])
        self.update_mask()

    def update_mask(self, q=None):
        self.mask = self.base_mask.copy()
        for roi in self.ROIs:
            self._apply_roi(roi[0], roi[1])

        self._apply_bounding_mask()

        self.image.raw_image = self.data.copy()
        self.image.raw_image[self.mask == 1] = self.image.raw_image.max()
        self.image.update_image()

    def _apply_bounding_mask(self):
        self.mask[:self.ui.xMinBox.value(), :] = 1
        self.mask[:, :self.ui.yMinBox.value()] = 1
        self.mask[self.ui.xMaxBox.value():, :] = 1
        self.mask[:, self.ui.yMaxBox.value():] = 1

    def _apply_roi(self, roi, val):
        _, coords = roi.getArrayRegion(np.ones_like(self.image.raw_image),
                                       self.image.imageItem, axes=(0, 1),
                                       returnMappedCoords=True)
        x = np.round(coords[0].ravel()).astype(int)
        y = np.round(coords[1].ravel()).astype(int)
        c, r = self.mask.shape
        valid = (x < c) & (x >= 0) & (y < r) & (y >= 0)
        self.mask[x[valid], y[valid]] = val

    def clear_mask(self, q=None, clear_base=True):
        if clear_base:
            self.base_mask = np.zeros_like(self.data)
        while len(self.ROIs) > 0:
            r = self.ROIs[0][0]
            self.image.imageViewBox.removeItem(r)
            del(self.ROIs[0])
        self.ui.yMinBox.setValue(0)
        self.ui.yMaxBox.setValue(self.ui.yMaxBox.maximum())
        self.ui.xMinBox.setValue(0)
        self.ui.xMaxBox.setValue(self.ui.xMaxBox.maximum())
        self.update_mask()





# -*- coding: utf-8 -*-
"""
@author: walroth
"""

# Standard library imports
import os
import json

# Other imports
import numpy as np
import pandas as pd
from pyFAI.units import Unit

# Qt imports
import pyqtgraph as pg
from pyqtgraph import Qt
from pyqtgraph.Point import Point
from pyqtgraph.Qt import QtCore, QtGui, QtWidgets
from pyqtgraph.parametertree.ParameterItem import ParameterItem
from pyqtgraph.parametertree.Parameter import Parameter

# This module imports
from ..gui_utils import XdartDecoder, XdartEncoder

class defaultWidget(Qt.QtWidgets.QWidget):
    """Pop up window for displaying default values for a parameterTree.
    
    attributes:
        layout: QGridLayout, widget layout
        openButton: QPushButton, opens default file
        parameters: pyqtgraph Parameters
        saveButton: QPushButton, saves to json file
        tree: pyqtgraph ParameterTree
    
    methods:
        load_defaults: Loads in a json file with defaults
        param_to_valdict: Prepares parameters for saving by converting
            them to dictionary
        save_defaults: Saves the default values to json file
        set_all_defaults: Makes the current values the default values
        set_defaults: Sets the default values using a provided dict
        set_parameters: Sets the list of parameters
    """
    sigSetUserDefaults = Qt.QtCore.Signal()

    def __init__(self, parameters=None, parent=None):
        """parameters: dict, parameters to use
        """
        super().__init__(parent)
        self.parameters = {}
        self.tree = pg.parametertree.ParameterTree()
        if parameters is not None:
            self.set_parameters(parameters)
        self.layout = Qt.QtWidgets.QGridLayout(self)
        self.setLayout(self.layout)
        self.layout.addWidget(self.tree, 0, 0, 1, 2)
        self.saveButton = Qt.QtWidgets.QPushButton()
        self.saveButton.clicked.connect(self.save_defaults)
        self.saveButton.setText("Save")
        self.layout.addWidget(self.saveButton, 1, 0)
        self.openButton = Qt.QtWidgets.QPushButton()
        self.openButton.clicked.connect(self.load_defaults)
        self.openButton.setText("Open")
        self.layout.addWidget(self.openButton, 1, 1)
    
    def set_parameters(self, parameters):
        """Sets the current parameters to the provided parameters.
        
        parameters: pyqtgraph Parameter, parameters to set
        """
        self.parameters = {}
        self.tree.clear()
        for param in parameters:
            self.parameters[param.name()] = param
            self.tree.addParameters(param)
    
    def param_to_valdict(self, param):
        """Converts parameters to dictionary
        
        args:
            param: pyqtgraph Parameter, value to be converted
        
        returns:
            valdict: dictionary of parameter values
        """
        valdict = {}
        if param.isType('group'):
            valdict[param.name()] = {}
            if param.hasChildren():
                for child in param.children():
                    valdict[param.name()].update(self.param_to_valdict(child))
        else:
            valdict[param.name()] = param.value()
        return valdict

    def set_defaults(self, param, valdict):
        """Sets the defaults of the parameters to the provided valdict
        values.
        
        args:
            param: pyqtgraph Parameter, value whose default will be set
            valdict: dict, values to be set as defaults.
        """
        if param.isType('group'):
            if param.hasChildren():
                for child in param.children():
                    self.set_defaults(child, valdict[param.name()])
        elif param.name() in valdict:
            param.setDefault(valdict[param.name()])
            param.setValue(valdict[param.name()])
    
    def save_defaults(self, checked=False, fname=None):
        """Opens a QFileDialog and saves the current values as json
        file.

        Parameters
        ----------
        fname : str, path to file to save defaults
        """
        emit = False
        if fname is None:
            fname, _ = Qt.QtWidgets.QFileDialog().getSaveFileName(filter="*.json")
            emit = True
        self.set_all_defaults()
        jdict = {}
        for key, param in self.parameters.items():
            jdict[key] = self.param_to_valdict(param)

        if fname != "":
            with open(fname, 'w') as f:
                json.dump(jdict, f, cls=XdartEncoder)
        if emit:
            self.sigSetUserDefaults.emit()
    
    def load_defaults(self, checked=False, fname=None):
        """Opens a QFileDialog and loads values from json file.

        Parameters
        ----------
        checked : bool, used by QAction triggered signal
        fname : str, path to file to load
        """
        emit = False
        if fname is None:
            fname, _ = Qt.QtWidgets.QFileDialog().getOpenFileName(filter="*.json")
            emit = True
        if fname != "":
            with open(fname, 'r') as f:
                valdict = json.load(f, cls=XdartDecoder)
            for key, param in self.parameters.items():
                try:
                    self.set_defaults(param, valdict[key])
                except KeyError:
                    print(f"Key Error in load_default, key: {key}")
        if emit:
            self.sigSetUserDefaults.emit()
    
    def set_all_defaults(self):
        """Sets the current values to be the default values for all
        parameters
        """
        for key, param in self.parameters.items():
            valdict = self.param_to_valdict(param)
            self.set_defaults(param, valdict)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'specUI.ui'
##
## Created by: Qt User Interface Compiler version 6.10.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
import os as _os
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFrame, QHBoxLayout, QLabel,
    QPushButton, QSizePolicy, QSpinBox, QVBoxLayout, QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(1440, 369)
        self.verticalLayout = QVBoxLayout(Form)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.paramFrame = QFrame(Form)
        self.paramFrame.setObjectName(u"paramFrame")
        self.paramFrame.setFrameShape(QFrame.StyledPanel)
        self.paramFrame.setFrameShadow(QFrame.Raised)

        self.verticalLayout.addWidget(self.paramFrame)

        self.specLabel = QLabel(Form)
        self.specLabel.setObjectName(u"specLabel")
        self.specLabel.setMinimumSize(QSize(0, 30))
        self.specLabel.setMaximumSize(QSize(16777215, 40))

        self.verticalLayout.addWidget(self.specLabel)

        self.commandFrame = QFrame(Form)
        self.commandFrame.setObjectName(u"commandFrame")
        self.commandFrame.setMinimumSize(QSize(0, 40))
        self.commandFrame.setMaximumSize(QSize(16777215, 40))
        self.commandFrame.setFrameShape(QFrame.StyledPanel)
        self.commandFrame.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.commandFrame)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(-1, 2, -1, 2)
        self.commandLayout = QHBoxLayout()
        self.commandLayout.setObjectName(u"commandLayout")
        self.advancedButton = QPushButton(self.commandFrame)
        self.advancedButton.setObjectName(u"advancedButton")
        self.advancedButton.setMaximumSize(QSize(80, 16777215))
        self.commandLayout.addWidget(self.advancedButton)
        self.modeLabel = QLabel(self.commandFrame)
        self.modeLabel.setObjectName(u"modeLabel")
        self.modeLabel.setVisible(False)  # hidden — no longer needed
        self.processingModeCombo = QComboBox(self.commandFrame)
        self.processingModeCombo.setObjectName(u"processingModeCombo")
        self.processingModeCombo.addItems([
            "Int 1D",
            "Int 2D",
            "Int 1D (XYE)",
            "Image Viewer",
            "XYE Viewer",
        ])
        self.commandLayout.addWidget(self.processingModeCombo)

        self.liveCheckBox = QCheckBox(self.commandFrame)
        self.liveCheckBox.setObjectName(u"liveCheckBox")
        self.liveCheckBox.setText(u"Live")
        self.commandLayout.addWidget(self.liveCheckBox)

        self.batchCheckBox = QCheckBox(self.commandFrame)
        self.batchCheckBox.setObjectName(u"batchCheckBox")
        self.batchCheckBox.setText(u"Batch")
        self.batchCheckBox.setChecked(True)
        self.commandLayout.addWidget(self.batchCheckBox)

        self.coresLabel = QLabel(self.commandFrame)
        self.coresLabel.setObjectName(u"coresLabel")
        self.commandLayout.addWidget(self.coresLabel)
        self.maxCoresSpinBox = QSpinBox(self.commandFrame)
        self.maxCoresSpinBox.setObjectName(u"maxCoresSpinBox")
        _cpu = _os.cpu_count() or 4
        self.maxCoresSpinBox.setMinimum(1)
        self.maxCoresSpinBox.setMaximum(_cpu)
        self.maxCoresSpinBox.setValue(min(_cpu - 1, 4) or 1)
        self.commandLayout.addWidget(self.maxCoresSpinBox)

        self.horizontalLayout_2.addLayout(self.commandLayout)


        self.verticalLayout.addWidget(self.commandFrame)

        self.frame = QFrame(Form)
        self.frame.setObjectName(u"frame")
        self.frame.setMaximumSize(QSize(16777215, 40))
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.horizontalLayout = QHBoxLayout(self.frame)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.startButton = QPushButton(self.frame)
        self.startButton.setObjectName(u"startButton")
        self.startButton.setFocusPolicy(Qt.ClickFocus)

        self.horizontalLayout.addWidget(self.startButton)

        self.pauseButton = QPushButton(self.frame)
        self.pauseButton.setObjectName(u"pauseButton")
        self.pauseButton.setFocusPolicy(Qt.ClickFocus)

        self.horizontalLayout.addWidget(self.pauseButton)

        self.continueButton = QPushButton(self.frame)
        self.continueButton.setObjectName(u"continueButton")
        self.continueButton.setFocusPolicy(Qt.ClickFocus)

        self.horizontalLayout.addWidget(self.continueButton)

        self.stopButton = QPushButton(self.frame)
        self.stopButton.setObjectName(u"stopButton")
        self.stopButton.setFocusPolicy(Qt.ClickFocus)

        self.horizontalLayout.addWidget(self.stopButton)


        self.verticalLayout.addWidget(self.frame)


        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Form", None))
        self.specLabel.setText("")
        self.advancedButton.setText(QCoreApplication.translate("Form", u"Advanced...", None))
        self.modeLabel.setText(QCoreApplication.translate("Form", u"Mode:", None))
        self.coresLabel.setText(QCoreApplication.translate("Form", u"Cores:", None))
        self.startButton.setText(QCoreApplication.translate("Form", u"Start", None))
        self.pauseButton.setText(QCoreApplication.translate("Form", u"Pause", None))
        self.continueButton.setText(QCoreApplication.translate("Form", u"Continue", None))
        self.stopButton.setText(QCoreApplication.translate("Form", u"Stop", None))
    # retranslateUi




# -*- coding: utf-8 -*-
"""
@author: walroth, vthampy
"""
__version__ = '0.4.2'
# Top level script for running gui based program

# Standard library imports
import sys
import gc
import os
import signal
import logging
import faulthandler
faulthandler.enable()  # Print Python traceback on bus error / segfault

# Set PySide6 as the Qt binding for pyqtgraph before any Qt imports.
# Also export MPLBACKEND so child processes (e.g. pyFAI-calib2) inherit it.
os.environ['PYQTGRAPH_QT_LIB'] = 'PySide6'
os.environ['MPLBACKEND'] = 'QtAgg'

# Set matplotlib backend before any matplotlib import can occur.
# Use QtAgg (the Qt6 backend) to match pyqtgraph's choice.
import matplotlib
matplotlib.use('QtAgg')

# Suppress pyFAI INFO logs (e.g. "No sensor configuration provided").
logging.getLogger('pyFAI').setLevel(logging.WARNING)
logging.getLogger('pyFAI.gui.matplotlib').setLevel(logging.ERROR)

logger = logging.getLogger(__name__)

# Qt imports
from typing import TYPE_CHECKING, Any
if TYPE_CHECKING:
    QtGui: Any = None
    QtWidgets: Any = None
else:
    from pyqtgraph.Qt import QtGui, QtWidgets

# This module imports
from xdart.gui.mainWindow import Ui_MainWindow
from xdart.gui import tabs


def setup_data_folders(exp_list):
    """
    Creates xdart/data folder and xdart/data/tabs folder for storing
    local data. These are ignored by gitignore.

    Parameters
    ----------
    exp_list : list, set of tabs to be

    Returns
    -------
    tab_paths : dict, paths for tabs to store data
    """
    current_directory = os.path.dirname(__file__)
    data_directory = os.path.join(current_directory, "../data")
    if not os.path.isdir(data_directory):
        os.mkdir(data_directory)
    tab_paths = {}
    for e in exp_list:
        tab_directory = os.path.join(data_directory, e)
        tab_paths[e] = tab_directory
        if not os.path.isdir(tab_directory):
            os.mkdir(tab_directory)
    return tab_paths


QMainWindow = QtWidgets.QMainWindow


class Main(QMainWindow):
    def __init__(self, tab_paths):
        """
        Parameters
        ----------
        tab_paths : dict
        """
        super().__init__()
        self.tab_paths = tab_paths
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.setWindowTitle('xdart')
        self.ui.actionOpen.triggered.connect(self.openFile)
        self.ui.actionExit.triggered.connect(self.exit)
        self.fname = None

        # Embed the main widget directly (no tab container)
        self.main_widget = tabs.static_scan.staticWidget(
            local_path=self.tab_paths['static_scan'])
        self.setCentralWidget(self.main_widget)

        self.show()
        self.resize(1600, 920)

    def exit(self):
        try:
            self.main_widget.close()
        finally:
            self.close()
            gc.collect()
            try:
                os.killpg(os.getpid(), signal.SIGTERM)
            except ProcessLookupError:
                pass
            sys.exit(0)

    def openFile(self):
        try:
            self.main_widget.open_file()
        except Exception:
            logger.exception("Error opening file")


def main():
    tab_paths = setup_data_folders(tabs.exp_list)
    app = QtWidgets.QApplication(sys.argv)
    mw = Main(tab_paths)
    mw.show()
    app.exec()


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-

# Standard Library imports
import time
import os
import unittest
import traceback

# Other imports
import numpy as np
import pyFAI
from pyFAI import units
import h5py

# add xdart to path
import sys
from pathlib import Path
if __name__ == "__main__":
    from config import xdart_dir
else:
    from .config import xdart_dir

if xdart_dir not in sys.path:
    sys.path.append(xdart_dir)

from xdart.modules.ewald import EwaldArch, EwaldSphere
from xdart.utils.containers import PONI

# Skip all tests if the test data directory does not exist on this machine.
_TEST_DATA_DIR = Path(xdart_dir) / "tests" / "test_data" / "spec_pd100k"
_SKIP_REASON = f"Test data not found: {_TEST_DATA_DIR}"

def read_RAW(file, mask = True):
    #print("Reading RAW file here...")
    try:
        with open(file, 'rb') as im:
            arr = np.frombuffer(im.read(), dtype='int32').copy()
        arr.shape = (195, 487)
        #arr = np.fliplr(arr)               # for the way mounted at BL2-1
        if mask:
            for i in range(0, 10):
                arr[:,i] = -2.0
            for i in range(477, 487):
                arr[:,i] = -2.0
        return arr
    except:
        traceback.print_exc()
        print("Error reading file: %s" % file)
        return None

def SPECread(filename, scan_number):
    print("Reading SPEC file here...")
    tth = []
    i0 = []
    spec = open(filename)
    for line in spec:
        if "#O" in line and "TwoTheta" in line: 
            temp = line.split()
            tth_line = temp[0][2]
            for i in range(0, len(temp)):
                if temp[i] == "TwoTheta":	
                    tth_pos = i
                    break
            break
    for line in spec:
        if "#S" in line:
            temp = line.split()
            if int(temp[1]) == scan_number:
                break
    for line in spec:
        if "#P" + str(tth_line) in line:
            temp = line.split()
            tth_start = float(temp[tth_pos])
            break
    for line in spec:
        if "#L" in line:
            motors = line.split()[1:]
            if "TwoTheta" not in line:
                tth_motor_bool = False
                print("2theta is not scanned...")
            else:
                tth_motor_bool = True
                tth_motor = motors.index("TwoTheta")
            i0_motor = motors.index("Monitor")
            break
    for line in spec:
        try:
            temp = line.split()
            if tth_motor_bool:
                tth = np.append(tth, float(temp[tth_motor]))
            else:
                tth = np.append(tth, tth_start)
            i0 = np.append(i0, float(temp[i0_motor]))
        except:
            break
    spec.close()
    return tth, i0


def make_sphere(calib_path, poni_file, stepsize, user, image_path, spec_path, 
                spec_name, scan_number):
    tth, i0 = SPECread(spec_path + spec_name, scan_number)
    xmax_global = 0.0  
    xmin_global = 180.0
    times = {'overall':[], 'creation':[], 'int':[], 'add':[]}
    detector = pyFAI.detectors.Detector(172e-6, 172e-6)
    data_file_path = os.path.join(xdart_dir, "tests/test_data/spec_pd100k/test_save.h5")
    sphere = EwaldSphere(data_file=data_file_path)
    sphere.save_to_h5(replace=True)
    poni = PONI.from_poni_file(os.path.join(calib_path, poni_file))
    for k in range(1, len(tth)):
        start = time.time()
        filename = (image_path + user + spec_name + "_scan" + str(scan_number) +
                    "_" + str(k).zfill(4) + ".raw")
        map_raw = read_RAW(filename)
        map_raw = map_raw.T
        mask = np.where(map_raw < 0, 1, 0)
        rot2 = np.radians(-tth[k])
        poni.rot2 = rot2
        start_c = time.time()
        arch = EwaldArch(idx=k, map_raw=map_raw, poni=poni, scan_info={'i0':i0[k]})
        times['creation'].append(time.time() - start_c)
        start_i = time.time()
        arch.integrate_1d(numpoints=18000, monitor='i0', radial_range=[0,180], 
                        unit=units.TTH_DEG, correctSolidAngle=False, 
                        method='csr')
        arch.integrate_2d(npt_rad=1000, npt_azim=1000, monitor='i0', radial_range=[0,30], azimuth_range=[70,110], method='csr')
        times['int'].append(time.time() - start_i)
        start_a = time.time()
        sphere.add_arch(arch.copy(), calculate=False, set_mg=False)
        sphere.save_to_h5(arches=[k], data_only=True, replace=False)
        times['add'].append(time.time() - start_a)
        
        times['overall'].append(time.time() - start)
    return sphere


@unittest.skipUnless(_TEST_DATA_DIR.exists(), _SKIP_REASON)
class TestEwaldSphere(unittest.TestCase):
    def setUp(self):
        self.sphere = make_sphere(
            calib_path=os.path.join(xdart_dir, "tests/test_data/spec_pd100k/"),
            poni_file="poni.poni",
            stepsize=0.01,
            user="b_stone_",
            image_path=os.path.join(xdart_dir, "tests/test_data/spec_pd100k/images/"),
            spec_path=os.path.join(xdart_dir, "tests/test_data/spec_pd100k/"),
            spec_name="LaB6_2",
            scan_number=1
        )
        
        self.true_1d_ttheta = np.load(
            os.path.join(xdart_dir, "tests/test_data/spec_pd100k/1d_ttheta.npy")
        )
        self.true_1d_norm = np.load(
            os.path.join(xdart_dir, "tests/test_data/spec_pd100k/1d_norm.npy")
        )
        
        self.true_2d_ttheta = np.load(
            os.path.join(xdart_dir, "tests/test_data/spec_pd100k/2d_ttheta.npy")
        )
        self.true_2d_norm = np.load(
            os.path.join(xdart_dir, "tests/test_data/spec_pd100k/2d_norm.npy")
        )
        self.true_2d_chi = np.load(
            os.path.join(xdart_dir, "tests/test_data/spec_pd100k/2d_chi.npy")
        )
    
    #@unittest.skip("Known to pass")
    def test_1d(self):
        self.assertTrue(np.isclose(self.true_1d_ttheta, 
                                   self.sphere.bai_1d.ttheta).all())
        self.assertTrue(np.isclose(self.true_1d_norm, 
                                   self.sphere.bai_1d.norm.full(),
                                   rtol=1e-3, atol=1e-4).all())
    
    #@unittest.skip("Known to pass")
    def test_2d(self):
        self.assertTrue(np.isclose(self.true_2d_ttheta, 
                                   self.sphere.bai_2d.ttheta).all())
        self.assertTrue(np.isclose(self.true_2d_norm, 
                                   self.sphere.bai_2d.norm.full(),
                                   rtol=1e-3, atol=1e-4).all())
        self.assertTrue(np.isclose(self.true_2d_chi, 
                                   self.sphere.bai_2d.chi).all())
    
    def test_save(self):
        self.sphere.save_to_h5(arches=[])
        self.sphere.save_to_h5(arches=[1], data_only=True, replace=False)
    
    def test_bai(self):
        self.sphere.by_arch_integrate_1d(numpoints=18000, monitor='i0', radial_range=[0,180], 
                        unit=units.TTH_DEG, correctSolidAngle=False, 
                        method='csr')
        self.assertEqual(self.sphere.bai_1d_args['method'], 'csr')
        self.assertTrue(np.isclose(self.true_1d_norm, 
                                   self.sphere.bai_1d.norm.full(),
                                   rtol=1e-3, atol=1e-4).all())


if __name__ == '__main__':
    unittest.main()
    # sphere = make_sphere(
    #         calib_path="test_data/spec_pd100k/",
    #         poni_file="poni.poni",
    #         stepsize=0.01,
    #         user="b_stone_",
    #         image_path="test_data/spec_pd100k/images/",
    #         spec_path="test_data/spec_pd100k/",
    #         spec_name = "LaB6_2",
    #         scan_number = 1
    #     )




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'imageWidgetUI.ui'
##
## Created by: Qt User Interface Compiler version 6.10.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QFrame, QGridLayout,
    QHBoxLayout, QSizePolicy, QToolButton, QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(688, 480)
        self.horizontalLayout = QHBoxLayout(Form)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.imageFrame = QFrame(Form)
        self.imageFrame.setObjectName(u"imageFrame")
        self.imageFrame.setFrameShape(QFrame.NoFrame)
        self.imageFrame.setFrameShadow(QFrame.Plain)
        self.imageFrame.setLineWidth(0)

        self.horizontalLayout.addWidget(self.imageFrame)

        self.toolFrame = QFrame(Form)
        self.toolFrame.setObjectName(u"toolFrame")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolFrame.sizePolicy().hasHeightForWidth())
        self.toolFrame.setSizePolicy(sizePolicy)
        self.toolFrame.setMinimumSize(QSize(120, 0))
        self.toolFrame.setFrameShape(QFrame.NoFrame)
        self.toolFrame.setFrameShadow(QFrame.Plain)
        self.toolFrame.setLineWidth(0)
        self.toolLayout = QGridLayout(self.toolFrame)
        self.toolLayout.setObjectName(u"toolLayout")
        self.toolLayout.setContentsMargins(1, 1, 1, 1)
        self.cmapBox = QComboBox(self.toolFrame)
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.addItem("")
        self.cmapBox.setObjectName(u"cmapBox")

        self.toolLayout.addWidget(self.cmapBox, 0, 1, 1, 1)

        self.logButton = QToolButton(self.toolFrame)
        self.logButton.setObjectName(u"logButton")
        self.logButton.setMinimumSize(QSize(30, 0))
        self.logButton.setMaximumSize(QSize(30, 16777215))
        self.logButton.setCheckable(True)

        self.toolLayout.addWidget(self.logButton, 0, 0, 1, 1)


        self.horizontalLayout.addWidget(self.toolFrame)


        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Form", None))
        self.cmapBox.setItemText(0, QCoreApplication.translate("Form", u"grey", None))
        self.cmapBox.setItemText(1, QCoreApplication.translate("Form", u"viridis", None))
        self.cmapBox.setItemText(2, QCoreApplication.translate("Form", u"plasma", None))
        self.cmapBox.setItemText(3, QCoreApplication.translate("Form", u"inferno", None))
        self.cmapBox.setItemText(4, QCoreApplication.translate("Form", u"magma", None))
        self.cmapBox.setItemText(5, QCoreApplication.translate("Form", u"spectrum", None))
        self.cmapBox.setItemText(6, QCoreApplication.translate("Form", u"thermal", None))
        self.cmapBox.setItemText(7, QCoreApplication.translate("Form", u"flame", None))
        self.cmapBox.setItemText(8, QCoreApplication.translate("Form", u"yellowy", None))
        self.cmapBox.setItemText(9, QCoreApplication.translate("Form", u"bipolar", None))
        self.cmapBox.setItemText(10, QCoreApplication.translate("Form", u"cyclic", None))
        self.cmapBox.setItemText(11, QCoreApplication.translate("Form", u"greyclip", None))

        self.logButton.setText(QCoreApplication.translate("Form", u"Log", None))
    # retranslateUi




# -*- coding: utf-8 -*-
"""
@author: walroth
"""

# Standard library imports
import os

# Other imports
import numpy as np
from pathlib import Path
from collections import OrderedDict

import fabio

# Detector File Sizes
detector_file_sizes = {
    'Rayonix MX225': 18878464,
    'Rayonix SX165': 8392704,
    'Pilatus 100k': 379860,
    'Pilatus 1M': 4092732,
}

# ---------------------------------------------------------------------------
# I/O helpers — re-exported from ssrl_xrd_tools
# ---------------------------------------------------------------------------
from ssrl_xrd_tools.io.export import write_xye  # noqa: E402, F401

# ---------------------------------------------------------------------------
# HDF5 codec — imported from ssrl_xrd_tools.core.hdf5
# ---------------------------------------------------------------------------
from ssrl_xrd_tools.core.hdf5 import (  # noqa: E402
    check_encoded,
    soft_list_eval,
    catch_h5py_file,
    data_to_h5,
    none_to_h5,
    dict_to_h5,
    str_to_h5,
    scalar_to_h5,
    arr_to_h5,
    series_to_h5,
    dataframe_to_h5,
    index_to_h5,
    encoded_h5,
    attributes_to_h5,
    h5_to_data,
    h5_to_index,
    h5_to_dict,
    h5_to_attributes,
)


def get_fname_dir():
    """
    Returns directory on local drive to save temporary h5 files in.

    Returns:
        path: {str} Path where h5 file is saved
    """
    home_path = str(Path.home())
    path = os.path.join(home_path, 'xdart_processed_data')
    Path(path).mkdir(parents=True, exist_ok=True)
    return path


def split_file_name(fname):
    """Splits filename to get directory, file root and extension.

    Arguments:
        fname {str} -- full image file name with path
    """
    directory = os.path.dirname(fname)
    root, ext = os.path.splitext(os.path.basename(fname))

    if len(ext) > 0:
        if ext[0] == '.':
            ext = ext[1:]

    return directory, root, ext


def get_sname_img_number(fname):
    """Splits filename to get scan name and image number.

    Arguments:
        fname {str} -- full image file name with path
    Returns:
        series_name {str} -- series name (or root if single acquisition)
        img_number {int/None} -- image number (if part of series)
    """
    directory, root, ext = split_file_name(fname)
    try:
        img_number = int(root[root.rindex('_') + 1:])
        root = root[:root.rindex('_')]
    except ValueError:
        img_number = None

    return root, img_number


def get_series_avg(fname, detector, meta_ext):
    """Returns the averaged image and meta data for a series.

    Arguments:
        fname {str} -- full image file name with path
        detector {obj} -- pyFAI detector object
    Returns:
        series_name {str} -- series name (if series exists)
        series_file_names {str array} -- file names in series
        img_data {ndarray} -- averaged 2D intensity over a series
        img_meta {dict} -- averaged meta data over a series
    """
    fpath = Path(fname)
    series_name, img_number = get_sname_img_number(fname)
    if img_number is None:
        return None, None, None, None

    fnames = [str(f) for f in (fpath.parent.glob(f'{series_name}*[0-9][0-9][0-9][0-9]{fpath.suffix}'))]
    data, img_meta = 0, {}
    for ii, fname in enumerate(fnames):
        data += get_img_data(fname, detector, return_float=True)
        if data is None:
            return None, None, None, None

        meta = get_img_meta(fname, meta_ext) if meta_ext else {}
        if ii == 0:
            img_meta = meta
        else:
            for (k, v) in meta.items():
                try:
                    img_meta[k] += meta[k]
                except TypeError:
                    pass

    n = len(fnames)
    data /= n
    for (k, v) in img_meta.items():
        try:
            img_meta[k] /= n
        except TypeError:
            pass

    return series_name, fnames, data, img_meta


def match_img_detector(img_file, poni):
    """Check if the file is created by the detector specified by *poni*.

    Parameters
    ----------
    img_file : str | Path
        Path to the candidate image file.
    poni : ssrl_xrd_tools.core.containers.PONI
        Calibration object whose ``detector`` field is the pyFAI detector name.
    """
    if poni is None:
        return True
    detector_name = poni.detector if poni.detector else ''
    if detector_name not in detector_file_sizes:
        return True
    return os.stat(img_file).st_size == detector_file_sizes[detector_name]


def get_img_meta(img_file, meta_ext, spec_path=None, rv='all'):
    """Get image meta data from pdi/txt files for different beamlines.

    Delegates to ssrl_xrd_tools.io.metadata.read_image_metadata.

    Args:
        img_file (str): Image file for which meta data is required
        meta_ext (str): Extension of Meta file ('txt', 'pdi', or 'SPEC')
        spec_path (str): Unused; kept for backward compatibility
        rv (str, optional): Unused; kept for backward compatibility

    Returns:
        [dict]: Dictionary with all the meta data
    """
    from ssrl_xrd_tools.io.metadata import read_image_metadata
    fmt = 'spec' if (meta_ext or '').upper() == 'SPEC' else (meta_ext or 'txt')
    return read_image_metadata(img_file, meta_format=fmt)


def get_img_data(
        fname, detector, orientation='horizontal',
        flip=False, fliplr=False, transpose=False,
        return_float=False, im=0):
    """Read image file and return numpy array.

    Args:
        fname (str): File Name with path
        detector (detector object): pyFAI detector object
        orientation (str, optional): Orientation of detector.
            Options: 'horizontal', 'vertical'. Defaults to 'horizontal'.
        flip (bool, optional): Flip up-down. Defaults to False.
        fliplr (bool, optional): Flip left-right. Defaults to False.
        transpose (bool, optional): Transpose image. Defaults to False.
        return_float (bool, optional): Convert to float. Defaults to False.
        im (integer, optional): Frame number for multi-image files. Defaults to 0.

    Returns:
        ndarray: Image data read into numpy array
    """
    try:
        with fabio.open(fname) as f:
            if im == 0:
                img_data = f.data
            else:
                img_data = f.get_frame(im).data
    except Exception:
        # Fallback: raw binary for non-standard formats
        try:
            img_data = np.fromfile(fname, dtype='int32').reshape(detector.shape)
        except Exception:
            return None

    try:
        if img_data.shape != detector.shape:
            return None
    except AttributeError:
        return None

    if return_float:
        img_data = np.asarray(img_data, dtype=float)

    if (orientation == 'vertical') or transpose:
        img_data = img_data.T

    if flip:
        img_data = np.flipud(img_data)

    if fliplr:
        img_data = np.fliplr(img_data)

    return img_data


class FixSizeOrderedDict(OrderedDict):
    def __init__(self, *args, max=0, **kwargs):
        self._max = max
        super().__init__(*args, **kwargs)

    def __setitem__(self, key, value):
        if self._max > 0:
            if len(self) >= self._max:
                keys = list(self.keys())
                try:
                    k = int(key)
                    diffs = [abs(int(k_)-k) for k_ in keys]
                    out_key = keys[diffs.index(max(diffs))]
                    self.pop(out_key)
                except ValueError:
                    self.popitem(False)

        OrderedDict.__setitem__(self, key, value)
